// -*- c++ -*-

/*
 *
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef ACTIONS_H
#define ACTIONS_H

#include <qptrlist.h>
#include <qwidget.h>
#include <kaction.h>

class QSlider;
class QDockWindow;
class QLCDNumber;
class KMainWindow;

//
// Actions for displaying custom QWidgets.
//

/**
 * A KAction that displays a number using the @ref QLCDNumber class.
 *
 * @author Richard Moore, rich@kde.org
 * @version $Id: actions.h,v 1.10 2002/09/21 23:29:47 rich Exp $
 */
class LCDNumberAction : public KAction
{
    Q_OBJECT

public:
    /**
     * Creates a LCDNumberAction.
     *
     * @param text  The label for the LCD. Currently this is displayed
     *              as a tooltip.
     */
    LCDNumberAction( const QString &text, int accel,
		     QObject *receiver, const char *slot,
		     QObject *parent = 0, const char *name = 0 );

    virtual int plug( QWidget *widget, int index = -1 );
    virtual void unplug( QWidget *widget );

    int intValue() const { return val.toInt(); }
    QString value() const { return val; }

    int numDigits() const { return numDig; }

public slots:
    void display( int num );
    void display( const QString &s );

    void setNumDigits( int nDigits );

signals:
    void valueChanged( const QString & );

protected:
    virtual QLCDNumber *createWidget( QWidget *parent, const char *name=0 );

private:
    int numDig;
    QString val;
};

/**
 * A KAction that displays a number using a QSlider. Applications
 * can use this class to add a QSlider widget to their toolbars (eg. a
 * volume control).
 *
 * @author Richard Moore, rich@kde.org
 * @version $Id: actions.h,v 1.10 2002/09/21 23:29:47 rich Exp $
 */
class SliderAction : public KAction
{
    Q_OBJECT

public:
    SliderAction( int min, int max, int step, int val,
		  const QString &text,
		  QObject *parent = 0, const char *name = 0 );

    virtual int plug( QWidget *widget, int index = -1 );
    virtual void unplug( QWidget *widget );

    /**
     * Returns a pointer to the QSlider object.
     */
    QSlider *widget() const { return slide; }

    /**
     * Returns the current value of the slider.
     */
    int value() const;

    /**
     * A factory method that creates the QSlider object.
     */
    virtual QSlider *createWidget( QWidget *parent, const char *name=0 );

public slots:
    /**
     * Sets the value of the slider to the specified value.
     */
    void setValue( int num );

    /**
     * Sets the interval between the tickmarks of the slider.
     *
     * @ref QSlider::setTickInteral(int)
     */
    void setTickInterval( int ticks );

    /**
     * Sets the orientation of the slider.
     */
    void setOrientation( Orientation o );

    /**
     * Sets the orientation of the slider.
     */
    void dockChange( QDockWindow* );

signals:
    /**
     * Emitted when the value of the slider is changed. Note the 
     * orientation of the slider is automatically accounted for
     * when this signal is emitted.
     */
    void valueChanged( int );    

protected slots:
    /**
     * Sets the value of the slider action and emits @p valueChanged signal.
     *
     * @ref QSlider::setValue(int)
     */
    void setValueInternal( int num );

    /**
     * Same as above except it inverts the given value and calls 
     * @p setValueInternal.
     *
     * @ref #setValue(int)
     */
    void setInverseInternal( int num );
    
private:
    QSlider *slide;
    int min;
    int max;
    int step;
    int val;
    int tickStep;
};

/**
 * A KAction that hides and shows a widget.
 *
 * @author Richard Moore, rich@kde.org
 * @version $Id: actions.h,v 1.10 2002/09/21 23:29:47 rich Exp $
 */
class ToggleViewAction : public KRadioAction
{
    Q_OBJECT

public:
    /**
     * Creates a ToggleViewAction.
     */
    ToggleViewAction( const QString &text, const QString &icon,
		      const KShortcut &cut,
		      QObject *parent, const char *name );

    ~ToggleViewAction() {}

public slots:
    /**
     * Specifies the widget to be hidden and shown.
     */
    void setView( QWidget *w );

    /**
     * Updates the state of the action to reflect the visibility of the window.
     */
    void update();

    /**
     * Toggles the visibility of the view.
     */
    void toggleView();

    /**
     * Shows the view (if it was hidden).
     */
    void showView();

    /**
     * Hides the view (if it was visible).
     */
    void hideView();

protected slots:
    void slotActivated();

private:
    QWidget *win;
};

#endif // ACTIONS_H

