/*
 *
 * Copyright (C) 2002-2003 Rich Moore <rich@kde.org>
 * Copyright (C) 2002-2003 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qdir.h>
#include <qdom.h>
#include <qiconset.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qlayout.h>
#include <qpixmap.h>
#include <qpushbutton.h>
#include <qsplitter.h>
#include <qtimer.h>
#include <qtoolbutton.h>
#include <qtabwidget.h>
#include <qtextedit.h>
#include <qlabel.h>

#include <kaction.h>
#include <kconfig.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kkeydialog.h>
#include <klocale.h>
#include <kmenubar.h>
#include <kpopupmenu.h>
#include <krun.h>
#include <kshortcut.h>
#include <kstandarddirs.h>
#include <kstatusbar.h>
#include <ktoolbar.h>
#include <kurl.h>

#include <kwin.h>
#include <netwm.h>

#include "qtvisionwidget.h"
#include "viewmanager.h"
#include "qtvision.h"
#include "qtvisionactions.h"
#include "actions.h"
#include "sidebar.h"
#include "channelstore.h"
#include "qtvisionsystray.h"
#include "savercontrol.h"
#include "dpmscontrol.h"
#include "osdmanager.h"
#include "cfgdata.h"
#include "changelogdialog.h"
#include "checkupdatesdialog.h"
#include "statusmanager.h"


#define STATUSBAR_MAIN		0
#define STATUSBAR_PROGRAM	1

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#ifdef HAVE_LIRC
#include "lirc.h"
#endif // HAVE_LIRC

#include "mainwindow.h"
#include "mainwindow.moc"

MainWindow::MainWindow( QWidget *parent, const char *name, WFlags f )
           :KMainWindow( parent, name ? name : "main_window", f ), 
           _systray(0), _lirc(0), _sleepdelay(0)
{
    // Views
    _view = new QSplitter( this );
    _side = new Sidebar( _view );
    _screen = new QtVisionWidget( _view );

    setCentralWidget( _view );

    // Create the sidebar
    _side->setScreen( _screen );
    _side->setFocusProxy(_screen);
    _side->createViews();

    // Create the actions
    actionCollection()->setHighlightingEnabled( true );
    createActions();

    // Apply main window settings & size
    restoreWindowSize( _screen->driver()->configHandle() );

    // Setup status messages
    connect( actionCollection(), SIGNAL( actionStatusText(const QString &) ),
             this, SLOT( statusMessage(const QString & ) ) );
    connect( actionCollection(), SIGNAL( clearStatusText() ),
             this, SLOT( clearStatusBar() ) );
#if 0    
    connect( _screen->driver(), SIGNAL( channelChanged(const QString &) ),
             statusBar(), SLOT( permanent(const QString & ) ) );
#endif

    connect( _screen, SIGNAL( showVideoDesktop(bool) ), this, SLOT( setShowTrayIcon(bool) ) );

    connect( _screen->driver(), SIGNAL( channelChanged(const QString &) ),
             this, SLOT( setCaption(const QString & ) ) );  
    
    connect( _screen->driver()->views(), SIGNAL( stayOnTopChanged(bool) ),
             this, SLOT( stayOnTopCfgChanged(bool) ) );
    connect( _screen->driver()->views(), SIGNAL( disableScreenSaver(bool, int) ),

             this, SLOT( screenSaverCfgChanged(bool, int) ) );
    connect( _screen->view(), SIGNAL( visibilityChanged( bool ) ),
             this, SLOT( screenSaverVisible( bool ) ) );
    connect( this, SIGNAL( numberKeyPressed(int) ),
             _screen->driver(), SLOT( processNumberKeyEvent(int) ) );

    // Enable/disable actions
    _actions->deviceChanged( 0 );

    _sleeptimer = new QTimer(parent);
    connect(_sleeptimer, SIGNAL(timeout()), this, SLOT(slotSleepTimerTimeout()));

    connect(this, SIGNAL(sleepTimerDelay(const QString &)),
	_screen->driver()->osdManager(), SLOT(displayCC(const QString &)));
    
    // use kdelirc instead
    // setupInfraRed();

    // Create the main window shell
    setXMLFile( "qtvisionui.rc" );
    createGUI();

    KMenuBar *mb = menuBar();
    mb->changeItem( mb->idAt(0), i18n("&Channels") );

    // Setup toolbar toggles
    KToolBar *volBar = toolBar( "volumeToolBar" );
    if ( volBar )
        _actShowVolbar->setView( volBar );

    KToolBar *viewBar = toolBar( "viewToolBar" );
    if ( viewBar )
        _actShowViewbar->setView( viewBar );

    // Create context menu
    insertContextMenu( "screen_context_popup" );

    // Store the current status of the screensaver
    _screenSaverWasEnabled = SaverControl::getSaverStatus();
    // and the dpms
    _dpmsWasEnabled = DpmsControl::getDpmsStatus();

    // Check the screensaver configuration & act accordingly
    screenSaverCfgChanged( _screen->driver()->config()->disableScreenSaver,
                           _screen->driver()->config()->screenSaverMode );

    // set stayOnTop according to the config
    stayOnTopCfgChanged( _screen->driver()->config()->stayOnTop );

    // Create the status bar
    statusBar()->insertItem("", STATUSBAR_MAIN, 1);
    statusBar()->setItemAlignment(STATUSBAR_MAIN, AlignLeft | AlignVCenter);
    statusBar()->insertItem("", STATUSBAR_PROGRAM, 0, true);

    connect(_screen->driver()->statusManager(),
		SIGNAL(messageSent(const QString&)),
		this, SLOT(shortMessage(const QString&)));

    connect(_screen->driver()->statusManager(),
		SIGNAL(programChanged(const QString&)),
		this, SLOT(sbProgramName(const QString&)));

    // allow installation of plugin gui elements
    _screen->driver()->setGuiFactory(guiFactory(), actionCollection());

    QTimer::singleShot( 10, _screen->driver(), SLOT(start()) );
}

MainWindow::~MainWindow()
{
    // If the screensaver was previously enabled, re-enable it.
    // May cause confusion if the screensaver's configuration
    // has been changed while this application was running...
    disableScreenSaver( false );
    
    // Save the current window settings & size    
    saveWindowSize( _screen->driver()->configHandle() );

    // So we don't crash on exit
    delete _actions;    
}

void MainWindow::createActions()
{
    // Standard actions
    (void) KStdAction::quit( qApp, SLOT( quit() ), actionCollection() );
    (void) KStdAction::showMenubar( this, SLOT( toggleMenubar() ), actionCollection() );
    (void) KStdAction::showToolbar( this, SLOT( toggleToolbar() ), actionCollection() );
    (void) KStdAction::showStatusbar( this, SLOT( toggleStatus() ), actionCollection() );

    // QtVision actions
    _actions = new QtVisionActions( _screen->driver() );
    _actions->createActions( _screen, actionCollection() );

    connect( _screen, SIGNAL( viewModeChanged(int) ), this, SLOT( checkViewMode(int) ) );
    connect( _screen, SIGNAL( frameCloseEvent() ), action("file_quit"), SLOT( activate() ));
    connect( _screen->driver(), SIGNAL( channelText(const QString &) ),
             _actions->channel(), SLOT( display(const QString &) ) );

    connect( _actions, SIGNAL(launchBindings()), this, SLOT(keyBindings()) );

    // View actions
    _actShowSidebar = new ToggleViewAction( i18n("Show Sidebar"), "view_sidetree", Key_F9,
                                           actionCollection(), "show_sidebar" );

    _actShowSidebar->setStatusText( i18n( "Shows and hides the sidebar." ) );
    _actShowSidebar->setView( _side );

    _actShowViewbar = new ToggleViewAction( i18n("Show View Toolbar"), "wizard", 0,
                                           actionCollection(), "show_view_toolbar" );
    _actShowViewbar->setChecked( true );

    _actShowVolbar = new ToggleViewAction( i18n("Show Volume Toolbar"), "ktv_volup", 0,
                                          actionCollection(), "show_volume_toolbar" );
    _actShowVolbar->setChecked( true );

    // Help actions
    _actChangeLog = new KAction( i18n("&Change Log..."), "log", 0,
                                  this, SLOT( showChangeLog() ),
                                  actionCollection(), "help_view_changelog" );

    _actCVSLog = new KAction( i18n("C&VS Change Log..."), "html", 0,
                               this, SLOT( showCVSLog() ),
                               actionCollection(), "help_view_cvslog" );
    _actWebSite = new KAction( i18n("KWinTV Web Site..."), "html", 0,
                                this, SLOT( showWebSite() ),
                                actionCollection(), "help_view_website" );

    _sleepTime = new KToggleAction( i18n("Sleep Timer"), "ktv_sleeptimer", Key_T,
                                 this, SLOT( sleepTimer() ), actionCollection(), 
                                 "sleep_timer" );
}

void MainWindow::setupInfraRed()
{
#ifdef HAVE_LIRC
    
    // Remove the instance in case this function gets invoked
    // twice.  This should not happen.  However, since we can
    // not be certain about that it is always better to be safe
    // than sorry...
    if (_lirc != 0)
      delete _lirc;
    
    // Attach it to this window, not its parent.  Actually
    // parent () in this case will return 0L since this is
    // the top-level window...
    _lirc = new Lirc (this, "qtvision");
    
    QMap<QString, QString> lircKeyMap;
    lircKeyMap["CH+"] = "channel_up";
    lircKeyMap["CH-"] = "channel_down";
    lircKeyMap["VOL+"] = "volume_up";
    lircKeyMap["VOL-"] = "volume_down";
    lircKeyMap["CH_UP"] = "channel_up";
    lircKeyMap["CH_DOWN"] = "channel_down";
    lircKeyMap["VOL_UP"] = "volume_up";
    lircKeyMap["VOL_DOWN"] = "volume_down";
    lircKeyMap["MUTE"] = "volume_mute";
    lircKeyMap["QUIT"] = "file_quit";
    lircKeyMap["POWER"] = "file_quit";
    //(*lircKeyMap)["FULL_SCREEN"] = "next_view_mode";
    //(*lircKeyMap)["FULLSCREEN"] = "next_view_mode";
    lircKeyMap["FULL_SCREEN"] = "toggle_fullscreen";
    lircKeyMap["FULLSCREEN"] = "toggle_fullscreen";
    lircKeyMap["TTX"] = "toggle_teletext";
    lircKeyMap["CC"] = "toggle_teletext";
    
    _lirc->setDefaultKeyMap(lircKeyMap);
    connect(_lirc, SIGNAL(command(const QString&, unsigned)),
                  SLOT(lircCommand(const QString&, unsigned)));
    connect(_lirc, SIGNAL(event(const QString&, unsigned)),
                  SLOT(lircCommand(const QString&, unsigned)));
#else
    kdWarning() << "MainWindow::setupInfraRed(): Lirc not available" << endl;
#endif // HAVE_LIRC
}

void MainWindow::checkViewMode( int mode )
{
    screenSaverModeChanged( _screen->driver()->config()->screenSaverMode );

    // show systray icon in video desktop mode and top level mode
    setShowTrayIcon( mode == QtVisionWidget::ViewVideoDesktop ||
	    mode == QtVisionWidget::ViewTopLevel );

    if ( _screen->isShowNormal() )
        show();
    else
        hide();
}

KPopupMenu *MainWindow::createMenu( const QString &name ) 
{
    KPopupMenu *pop = static_cast<KPopupMenu *>( factory()->container(name, this) );
    if ( !pop )
        return 0;

    pop->insertTitle( i18n("QtVision"), -1, 0 );
    return pop;
}

KPopupMenu *MainWindow::insertContextMenu( const QString &name ) 
{
    KPopupMenu *pop = createMenu( name );
    _screen->addContextMenu( pop );

    return pop;
}

void MainWindow::toggleStatus()
{
    if ( statusBar()->isVisible())
        statusBar()->hide();
    else
        statusBar()->show();
}

void MainWindow::toggleMenubar()
{
    if ( menuBar()->isVisible() )
        menuBar()->hide();
    else
        menuBar()->show();
}

void MainWindow::toggleToolbar()
{
    if ( toolBar()->isVisible() )
      toolBar()->hide();
    else
      toolBar()->show();
}

void MainWindow::setShowTrayIcon( bool show )
{
    if ( show ) {
        if ( !_systray ) {
            kdDebug() << "Creating systray icon" << endl;
            _systray = new QtVisionSysTray(this, "qtvision_systray", _actions);
            connect(_systray, SIGNAL(doubleClicked()), _screen, SLOT(resetViewMode()));
            connect(_systray, SIGNAL(leftClicked()), _screen, SLOT(bringToFront()));
        }
        kdDebug() << "Showing systray icon" << endl;
        _systray->show();
    }
    else {
        if ( _systray ) {
            _systray->hide();
            delete _systray;
            _systray = 0L;
        }
    }
}

void MainWindow::disableScreenSaver( bool disable )
{
    if ( disable ) {
        SaverControl::disableSaver();
        DpmsControl::disableDpms();
    }
    else {
        if ( _screenSaverWasEnabled )
            SaverControl::enableSaver();
        
        if ( _dpmsWasEnabled )
            DpmsControl::enableDpms();
    }
}

void MainWindow::screenSaverCfgChanged( bool disable, int mode ) {
  _activateVisibility = false;
  if ( disable ) {
    screenSaverModeChanged( mode );
  } else {
    if ( _screenSaverWasEnabled ) 
        SaverControl::enableSaver();
  }
}

void MainWindow::screenSaverModeChanged( int mode )
{
    switch( mode ) {
    case SAVER_RUNNING:
        disableScreenSaver( true );
        break;
    case SAVER_VISIBLE:
        _activateVisibility = true;
        screenSaverVisible( _screen->view()->isVisible() );
        break;
    case SAVER_FULLSCREEN:
        if ( _screen && _screen->isShowFullScreen() ) {
            disableScreenSaver( true );
        } else {
            disableScreenSaver( false );
        }
        break;
    default:
        kdDebug() << "MainWindow: We should never get here!" << endl;
        break;
    }    
}

void MainWindow::screenSaverVisible( bool isVisible )
{
    if ( _activateVisibility ) {
        if ( isVisible ) {
            disableScreenSaver( true );
        } else {
            disableScreenSaver( false );
        }
    }
}

void MainWindow::saveWindowSize( KConfig *config )
{
    config->setGroup( "Window Geometry" );

    config->writeEntry( "Window Position X", x() );
    config->writeEntry( "Window Position Y", y() );
    config->writeEntry( "Window Width", width() );
    config->writeEntry( "Window Height", height() );
    config->writeEntry( "Splitter Position", _view->sizes() );    
    config->writeEntry( "Show Sidebar", !_side->isHidden() );
}

void MainWindow::restoreWindowSize( KConfig *config )
{
    config->setGroup( "Window Geometry" );

    int x = config->readNumEntry( "Window Position X", 0 );
    int y = config->readNumEntry( "Window Position Y", 0 );
    int width = config->readNumEntry( "Window Width", 550 );
    int height = config->readNumEntry( "Window Height", 400 );
    move( x, y );
    resize( width, height );

    QValueList<int> sizes = config->readIntListEntry( "Splitter Position" );
    if ( !sizes.count() ) {
        sizes.append( 20 );
        sizes.append( 80 );
    }
    _view->setSizes( sizes );

    bool sidebar = config->readBoolEntry( "Show Sidebar", true );
    if ( sidebar )
        _side->show();
    else
        _side->hide();
    
    _actShowSidebar->setChecked( sidebar );
}

void MainWindow::showChangeLog()
{
    ChangeLogDialog *dlg = new ChangeLogDialog( this, "changelog", false, 
                                                Qt::WType_Dialog | 
                                                Qt::WDestructiveClose );
    connect( dlg->closeButton, SIGNAL( clicked() ), dlg, SLOT( close() ) );

    QString log = locate("appdata", "ChangeLog");
    QFile file( log.isNull() ? QString("ChangeLog") : log );

    if ( file.open( IO_ReadOnly ) ) {
        QByteArray data = file.readAll();
        QString s( data );
        QFont f = dlg->edit->font();
        f.setFamily( "courier" );
        dlg->edit->setFont( f );
        dlg->edit->setText( s );
    }

    dlg->show();
}

void MainWindow::showCVSLog()
{
    KURL url( i18n("http://www.kwintv.org/rewrite_changelog.html") );
    KRun::runURL( url, QString("text/html") );
}

void MainWindow::showWebSite()
{
    KURL url( i18n("http://www.kwintv.org/") );
    KRun::runURL( url, QString("text/html") );
}

void MainWindow::lircCommand(const QString &cmd, unsigned /*repeat*/)
{
    if (cmd.length() == 1 && cmd[0] >= '0' && cmd[0] <= '9') {
        // handle numpad keys
        int key = cmd.toInt();
        emit numberKeyPressed(key);
    }

    KAction* act = action( cmd.latin1() );
    if (act)
      act->activate();
    else
        kdDebug() << "lirc: command '" << cmd << "' not recognized" << endl;
}

void MainWindow::keyBindings()
{
    KKeyDialog::configure(actionCollection());
}

void MainWindow::stayOnTopCfgChanged(bool enable)
{
    if (enable)
        KWin::setState(winId(), NET::StaysOnTop);
    else
        KWin::clearState(winId(), NET::StaysOnTop);
}

void MainWindow::sbProgramName(const QString& name) {
    statusBar()->changeItem(name, STATUSBAR_PROGRAM);
}

void MainWindow::statusMessage(const QString& name) {
    statusBar()->changeItem(name, STATUSBAR_MAIN);
}

void MainWindow::shortMessage(const QString& name) {
    statusBar()->changeItem(name, STATUSBAR_MAIN);
    QTimer::singleShot(2000, this, SLOT(clearStatusBar()));
}

void MainWindow::clearStatusBar() {
    statusBar()->changeItem("", STATUSBAR_MAIN);
}

void MainWindow::slotSleepTimerTimeout()
{
    kdDebug() << "QtVision: sleepTimer timeout reached - quitting client." << endl;
    QApplication::exit();
}

void MainWindow::sleepTimer()
{
    if (_sleeptimer->isActive())
        _sleeptimer->stop();

    kdDebug() << "sleepTimer() called.." << endl;

    if (_sleepdelay == 0) {
        // Change to 15 minutes.
        _sleepdelay = 900000;

    } else if (_sleepdelay == 900000) {
        // Change to 30 minutes.
        _sleepdelay = 1800000;

    } else if (_sleepdelay == 1800000) {
        // Change to 60 minutes.
        _sleepdelay = 3600000;

    } else if (_sleepdelay == 3600000) {
        // Change to 120 minutes.
        _sleepdelay = 7200000;

    } else {
        // Remove sleep-delay
        _sleepdelay = 0;
    }

    if (_sleepdelay)
          _sleeptimer->start(_sleepdelay, true);  // start the count-down timer

    // Display to the screen, how many minutes there are to the TV turns
    // off.
    emit sleepTimerDelay(QString::number(_sleepdelay / 1000 / 60));
}
