/*
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <qtimer.h>

#include <kaboutapplication.h>
#include <kaboutdata.h>
#include <kaction.h>
#include <kapp.h>
#include <kcmenumngr.h>
#include <kdebug.h>
#include <kglobal.h>
#include <klocale.h>
#include <kpopupmenu.h>
#include <kstddirs.h>
#include <kxmlguifactory.h>

#include <kparts/genericfactory.h>

#include "actions.h"
#include "qtvisionwidget.h"
#include "qtvisionactions.h"

#include "qtvisionpart.h"
#include "qtvisionpart.moc"

typedef KParts::GenericFactory<QtVisionPart> QtVisionPartFactory;
K_EXPORT_COMPONENT_FACTORY( libqtvisionpart,  QtVisionPartFactory );

QtVisionPart::QtVisionPart( QWidget *wparent, const char *wname,
			    QObject *parent, const char *name,
			    const QStringList &/*args*/ )
    : KParts::ReadOnlyPart( parent ? parent : wparent, name ? name : "qtvision_part" ),
      autoStart(true)
{
    kdDebug() << "Creating QtVisionPart" << endl;

    setInstance( QtVisionPartFactory::instance() );

    // Create TV Widget
    tv = new QtVisionWidget( wparent, wname ? wname : 0 );
    setWidget( tv );

    kdDebug() << "QtVisionPart: Created TV widget" << endl;

    // QtVision actions
    actions = new QtVisionActions( tv->driver() );
    actions->createActions( tv, actionCollection() );

    // Setup Action Collection
    actionCollection()->setHighlightingEnabled( true );

    // Define UI
    setXMLFile( "qtvisionpartui.rc" );

    QTimer::singleShot( 0, this, SLOT(maybeStart()) );
    kdDebug() << "QtVisionPart: Created" << endl;
}

QtVisionPart::~QtVisionPart()
{
}

void QtVisionPart::start()
{
    tv->driver()->start();
    addContextMenu( tv->view() );
}

void QtVisionPart::maybeStart()
{
    if ( autoStart )
	start();
}

KAboutData *QtVisionPart::createAboutData()
{
    // the non-i18n name here must be the same as the directory in
    // which the part's rc file is installed ('partrcdir' in the
    // Makefile)
    KAboutData *about = new KAboutData( "qtvision", I18N_NOOP("qtvision"),
				       "Development post-alpha 1",
				       I18N_NOOP("TV for KDE"),
				       KAboutData::License_LGPL,
				       I18N_NOOP("(c) 2002 The KWinTV Developers"),
					 0, "http://www.kwintv.org/");
    about->addAuthor("George Staikos", "Primary author and maintainer", "staikos@kde.org");
    about->addAuthor("Richard Moore", "GUI design, channel plugins", "rich@kde.org");
    about->addAuthor("Kevin Hessels", "Configuration dialog", "khessels@shaw.ca");
    about->addAuthor("Stefan Hellwig", "Webmaster, developer", "stefan@stefanhellwig.de");
    about->addAuthor("Rizsanyi Zsolt", "Infrared remote support, developer", "rizsanyi@myrealbox.com");
    about->addAuthor("Dawit Alemayehu", "Developer", "adawit@kde.org");

    return about;
}

void QtVisionPart::addContextMenu( bool xmlgui )
{
   tv->addContextMenu( createContextMenu( xmlgui ) );
}

KPopupMenu *QtVisionPart::createContextMenu( bool xmlgui ) 
{
    KPopupMenu *pop = 0;
    if ( xmlgui ) {
	pop = createContextMenu( createContextMenu(QString("screen_context_popup")) );
	if ( pop )
	    return pop;
    }

    // Fall back to manual menu creation
    pop = new KPopupMenu;

    actions->channel()->plug( pop );
    actions->channelUp()->plug( pop );
    actions->channelDown()->plug( pop );
    actions->volumeSlider()->plug( pop );
    actions->volumeMute()->plug( pop );
    pop->insertSeparator();

    actions->showNormal()->plug( pop );
    actions->showFullScreen()->plug( pop );
    actions->showTopLevel()->plug( pop );

    return pop;
}

KPopupMenu *QtVisionPart::createContextMenu( const QString &name ) 
{
    kdDebug() << "QtVisionPart: createContextMenu(...)" << endl;

    KXMLGUIFactory *fact = factory();
    if ( !fact )
        return 0;

    QWidget *w = fact->container( name.isNull() ? QString("screen_context_popup") : name, this );
    if ( !w )
        return 0;

    KPopupMenu *pop = static_cast<KPopupMenu *>( w );
    pop->insertTitle( i18n("QtVision"), -1, 0 );
    return pop;
}

bool QtVisionPart::openFile()
{
    kdDebug() << "QtVisionPart::openFile() '" << filename() << "'" << endl;

    kdDebug() << "QtVisionPart::openFile(): Extend QtVision interface to cover loading" << endl;

    return false;
}

void QtVisionPart::about()
{
    KAboutApplication dlg( createAboutData() );
    dlg.exec();
}

void QtVisionPart::help()
{
    KApplication::kApplication()->invokeHelp( QString::null, QString("qtvision") );
}


void QtVisionPart::preferences()
{
    QtVision *qtv = tv->driver();
    if ( qtv )
        qtv->settings();
}

void QtVisionPart::reportBug()
{
    KApplication::kApplication()->invokeMailer( QString("kwintv@mail.kde.org"), QString::null );
}
