/*
 *
 * Copyright (C) 2002 Stefan Hellwig <stefan@stefanhellwig.de>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <dcopclient.h>
#include <dcopobject.h>
#include <kapplication.h>
#include <kdebug.h>
#include <qdatastream.h>
  
#include "savercontrol.h"

SaverControl::SaverControl()
{
    kdDebug() << "SaverControl: Constructor called" << endl;
    // Check DCOP configuration facilities
    // Check if KDE screensaver is enabled
    getSaverDCOPIface();
    saverWasEnabled = isScreenSaverEnabled();
}


SaverControl::~SaverControl()
{
    // At the end: enable screensaver if it has been active on startup
    disableScreenSaver( false );
}

bool SaverControl::getSaverDCOPIface() {
    // Try to find the DCOP interface of the KDE screensaver. If it is found, we use
    // it to communicate with it in order to (de-)activate the KDE screensaver.

    dcop_KDESaver = kapp->dcopClient()->findObject( "kdesktop",
                                                    "KScreensaverIface",
                                                    "enable(bool)",
                                                    dcop_data,
                                                    dcop_screenSaverApp,
                                                    dcop_screenSaverObj );
    if ( dcop_KDESaver ) {
      return true;
    } else {
      return false;
    }
}

bool SaverControl::isScreenSaverEnabled() {

    // Use DCOP to communicate with the KDE screensaver. This way we can find
    // out if it is currently enabled. That's the more elegant way than to scan
    // the screensaver's configuration file.

    bool enabled = false;

    QByteArray data, replyData;
    QCString replyType;
    if ( !kapp->dcopClient()->call( dcop_screenSaverApp, dcop_screenSaverObj, "isEnabled()",
                                    data, replyType, replyData) ) {
      kdDebug() << "SaverControl: Error in DCOP communication. Unable to get status of KDE Screensaver" << endl;
    } else {
      QDataStream reply(replyData, IO_ReadOnly);
      if (replyType == "bool") {
        reply >> enabled;
        kdDebug() << "SaverControl: KDE's screensaver isEnabled() returned " << enabled << endl;
      } else {
        kdDebug() << "SaverControl: Error in DCOP communication. Unexpected return type." << endl;
      }
    }
    return enabled;
}


void SaverControl::disableScreenSaver( bool disable ) {
    if ( !dcop_KDESaver ) return;

    if ( disable ) {
        QByteArray data;
        QDataStream arg(data, IO_WriteOnly);
        arg << false;
        if ( !kapp->dcopClient()->send( dcop_screenSaverApp, dcop_screenSaverObj, "enable(bool)", data ) ) {
          kdDebug() << "SaverControl: Error in DCOP communication. Unable to disable KDE Screensaver." << endl;
        } else {
          kdDebug() << "SaverControl: KDE screensaver successfully disabled." << endl;
        }
    } else {
        if ( saverWasEnabled ) {
            QByteArray data;
            QDataStream arg(data, IO_WriteOnly);
            arg << true;
            if ( !kapp->dcopClient()->send( dcop_screenSaverApp, dcop_screenSaverObj, "enable(bool)", data ) ) {
              kdDebug() << "SaverControl: Error in DCOP communication. Unable to enable KDE Screensaver." << endl;
            } else {
              kdDebug() << "SaverControl: KDE screensaver successfully enabled." << endl;
            }
        }
    }

}

bool SaverControl::getSaverStatus() {

    bool        DCOP_KDESaver;
    QByteArray  DCOP_data;
    QCString    DCOP_screenSaverApp, DCOP_screenSaverObj;

    // Try to find the DCOP interface of the KDE screensaver. If it is found, we use
    // it to communicate with it in order to get the current setting of the screensaver.

    DCOP_KDESaver = kapp->dcopClient()->findObject( "kdesktop",
                                                      "KScreensaverIface",
                                                      "isEnabled()",
                                                      DCOP_data,
                                                      DCOP_screenSaverApp,
                                                      DCOP_screenSaverObj );
    if ( !DCOP_KDESaver ) return false;

    bool enabled = false;

    QByteArray data, replyData;
    QCString replyType;
    if ( !kapp->dcopClient()->call( DCOP_screenSaverApp, DCOP_screenSaverObj, "isEnabled()",
                                    data, replyType, replyData) ) {
      kdDebug() << "SaverControl: Error in DCOP communication. Unable to get status of KDE Screensaver" << endl;
    } else {
      QDataStream reply(replyData, IO_ReadOnly);
      if (replyType == "bool") {
        reply >> enabled;
      }
    }
    return enabled;
}

bool SaverControl::disableSaver() {

    bool        DCOP_KDESaver;
    QByteArray  DCOP_data;
    QCString    DCOP_screenSaverApp, DCOP_screenSaverObj;

    // Try to find the DCOP interface of the KDE screensaver. If it is found, we use
    // it to communicate with it in order to (de-)activate the KDE screensaver.

    DCOP_KDESaver = kapp->dcopClient()->findObject( "kdesktop",
                                                      "KScreensaverIface",
                                                      "enable(bool)",
                                                      DCOP_data,
                                                      DCOP_screenSaverApp,
                                                      DCOP_screenSaverObj );
    QByteArray data;
    QDataStream arg(data, IO_WriteOnly);
    arg << false;
    if ( !kapp->dcopClient()->send( DCOP_screenSaverApp, DCOP_screenSaverObj, "enable(bool)", data ) ) {
      kdDebug() << "SaverControl: Error in DCOP communication. Unable to disable KDE Screensaver." << endl;
      return false;
    } else {
      kdDebug() << "SaverControl: KDE screensaver successfully disabled." << endl;
      return true;
    }
}

bool SaverControl::enableSaver() {

    bool        DCOP_KDESaver;
    QByteArray  DCOP_data;
    QCString    DCOP_screenSaverApp, DCOP_screenSaverObj;

    // Try to find the DCOP interface of the KDE screensaver. If it is found, we use
    // it to communicate with it in order to (de-)activate the KDE screensaver.

    DCOP_KDESaver = kapp->dcopClient()->findObject( "kdesktop",
                                                      "KScreensaverIface",
                                                      "enable(bool)",
                                                      DCOP_data,
                                                      DCOP_screenSaverApp,
                                                      DCOP_screenSaverObj );
    QByteArray data;
    QDataStream arg(data, IO_WriteOnly);
    arg << true;
    if ( !kapp->dcopClient()->send( DCOP_screenSaverApp, DCOP_screenSaverObj, "enable(bool)", data ) ) {
      kdDebug() << "SaverControl: Error in DCOP communication. Unable to enable KDE Screensaver." << endl;
      return false;
    } else {
      kdDebug() << "SaverControl: KDE screensaver successfully enabled." << endl;
      return true;
    }
}

bool SaverControl::toggleSaver( bool enable ) {

  if( enable ) {
    return enableSaver();
  } else {
    return disableSaver();
  }
}
