/* 
   Copyright (c) 2002 Malte Starostik <malte@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// $Id: channelioxawtv.cpp,v 1.5 2003/05/20 05:51:26 staikos Exp $

#include <qtextstream.h>

#include <kdebug.h>

#include "channel.h"
#include "channelstore.h"

#include "channelioxawtv.h"
#include "frequencies.h"

struct ChannelIOFormatXawtv::ChannelInfo
{
	ChannelInfo() {}
	ChannelInfo( const QString& a_name ) : name( a_name ) {}
	QString name;
	QMap< QString, QString > params;
};

ChannelIOFormatXawtv::ChannelIOFormatXawtv(QtVision *qtv, QObject *parent, const char *name)
	: QVChannelPlugin(qtv, "XawTV Channels", parent, name)
{
	_fmtName = "xawtv";
	_flags = FormatRead;
}

bool ChannelIOFormatXawtv::load( ChannelStore* store, QIODevice* file, const QString& /* fmt */ )
{
	// xawtv's config file is in INI-style format, yet we can't use KConfig since
	// the order of sections determines the channel numbers. Sigh.
	QTextStream ts( file );
	QString section;
	// Store them here temporarily. Might be the [global] and/or [defatults]
	// sections come last in the file, so we can't translate channel => frequency
	// etc. before reading the whole file.
	QValueList< ChannelInfo > channels;
	QValueList< ChannelInfo >::Iterator channel = channels.end();
	this->store = store;
	while ( !ts.atEnd() )
	{
		QString line = ts.readLine().stripWhiteSpace();
		if ( line.isEmpty() || line[ 0 ] == '#' ) continue;
		if ( line[ 0 ] == '[' && line[ line.length() - 1 ] == ']' )
		{
			section = line.mid( 1, line.length() - 2 );
			if ( section == "defaults" || section == "global" || section == "launch" )
				channel = channels.end();
			else channel = channels.append( ChannelInfo( section ) );
			continue;
		}
		QStringList pair = QStringList::split( '=', line );
		if ( pair.count() != 2 ) continue;
		QString key = pair[ 0 ].stripWhiteSpace();
		QString value = pair[ 1 ].stripWhiteSpace();

		if ( channel == channels.end() )
		{
			if ( section == "global" )
			{
				if ( key == "freqtab" ) setFrequencyTable( value );
			}
		}
		else ( *channel ).params.replace( key, value );
	}

	for ( QValueList< ChannelInfo >::ConstIterator it = channels.begin(); it != channels.end(); ++it )
	{
		Channel *channel = new Channel( store );
		channel->setName( (*it).name );
		if ( !( *it ).params[ "freq" ].isEmpty() )
			channel->setFreq( ( *it ).params[ "freq" ].toULong() );
		else channel->setFreq( channelToFrequency( ( *it ).params[ "channel" ] ) );
		if ( !( *it ).params[ "fine" ].isEmpty() )
			channel->setFreq( channel->freq() + ( *it ).params[ "fine" ].toLong() );
		store->addChannel( channel );
	}
	return true;
}

bool ChannelIOFormatXawtv::save( ChannelStore* /* store */, QIODevice* /* file */, const QString& /* fmt */ )
{
	// TODO
	return false;
}

void ChannelIOFormatXawtv::setFrequencyTable( const QString& name )
{
	for ( CHANLISTS* list = chanlists; list->name; ++list )
		if ( name == list->name )
		{
			chanlist = list->list;
			chancount = list->count;
			return;
		}
	kdWarning() << "Unknown frequency table: " << name << endl;
}

unsigned long ChannelIOFormatXawtv::channelToFrequency( const QString& name )
{
	for ( int i = 0; i < chancount; ++i )
		if ( name == chanlist[ i ].name ) return chanlist[ i ].freq;
	kdWarning() << "Unknown channel name: " << name << endl;
	return 0;
}

extern "C" QVChannelPlugin* create_xawtvchannels(QtVision *qtv)
{
	return new ChannelIOFormatXawtv(qtv, 0L, "XawTV Channel Plugin");
}

// vim: ts=4 sw=4 noet
