/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <qwidget.h>
#include <klocale.h>
#include "simon.h"
#include <kdebug.h>
#include <kconfig.h>
#include <assert.h>
#include <qthread.h>
#include <qframe.h>
#include <qmutex.h>
#include <qevent.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qapplication.h>
#include <qsemaphore.h>
#include <qlayout.h>
#include <qcheckbox.h>
#include <qtvision.h>

#include <sys/types.h>
#include <unistd.h>
#include <stdlib.h>

extern "C" {
#include <libzvbi.h>
}


#include <osdmanager.h>

#include <statusmanager.h>


class CCEvent : public QEvent {
public:
	enum CCEventType { Unknown, CaptionText, NewTitle };
	CCEvent(CCEventType t = Unknown, const QString& text = "") 
		: QEvent(QEvent::User), _type(t), _text(text) {}
	virtual ~CCEvent() {}

	CCEventType _type;
	QString _text;

};


class CCThread : public QThread {
public:
	CCThread(QObject *owner, int pal, const QString& dev);
	virtual ~CCThread();
	virtual void run();
	void stop() { stopSem.tryAccess(1); }
	void channelChange() { chanChangeSem.tryAccess(1); }
	void captionEvent(vbi_event *e);
private:
	vbi_capture *_c;
	vbi_decoder *_d;
	vbi_sliced  *_sliced;
	unsigned int services;
	QObject *_owner;
	QSemaphore chanChangeSem;
	QSemaphore stopSem;
};


void caption_event(vbi_event *event, void *data) {
	static_cast<CCThread*>(data)->captionEvent(event);
}


CCThread::CCThread(QObject *owner, int pal, const QString& dev) 
: _owner(owner), chanChangeSem(1), stopSem(1)
{
int npal;

	if (pal == 0) { // NTSC
		services = VBI_SLICED_VBI_525
			| VBI_SLICED_CAPTION_525;
		npal = 525;
	} else {
		services = VBI_SLICED_VBI_625
			| VBI_SLICED_CAPTION_625
			| VBI_SLICED_WSS_625;
		npal = 625;
	}

	_c = vbi_capture_v4l2_new(const_cast<char*>(dev.latin1()), 5, &services, -1, 0, 0);

	if (!_c) {
		kdDebug() << "V4L2 vbi interface could not connect.  Trying v4l instead." << endl;
		_c = vbi_capture_v4l_new(const_cast<char*>(dev.latin1()), npal, &services, -1, 0, 0);
	}

	if (!_c) {
		kdDebug() << "VBI interface NOT established.  No captioning supported." << endl;
		return;
	}

	kdDebug() << "VBI interface established." << endl;
	_d = vbi_decoder_new();
	vbi_event_handler_register(_d,
				VBI_EVENT_CAPTION|VBI_EVENT_PROG_INFO,
				&caption_event,this);
	vbi_raw_decoder *r = vbi_capture_parameters(_c);
	_sliced = new vbi_sliced[r->count[0] + r->count[1]];
}


CCThread::~CCThread() { 
	if (_c) {
		delete[] _sliced; 
		vbi_event_handler_unregister(_d,&caption_event,this);
		vbi_decoder_delete(_d);
		_d = 0;
		vbi_capture_delete(_c);
		_c = 0;
	}
}


void CCThread::captionEvent(vbi_event *e) {
	switch(e->type) {
	case VBI_EVENT_CAPTION:
		//kdDebug() << "Caption event!! page=" << e->ev.caption.pgno << endl;
		vbi_page page;
		if (vbi_fetch_cc_page(_d, &page, e->ev.caption.pgno, 1)) {
			char buf[1024];
			int rc = vbi_print_page(&page, buf, sizeof(buf), "ISO-8859-1", true, true);
			if (rc > 0) {
				buf[rc] = 0;
				QString nbuf = buf;
				nbuf = nbuf.stripWhiteSpace();
				QApplication::postEvent(_owner,
					new CCEvent(CCEvent::CaptionText,nbuf));
				//kdDebug() << "[" << nbuf << "]" << endl;
			}
			vbi_unref_page(&page);
		}
		break;
	case VBI_EVENT_PROG_INFO:
		//kdDebug() << "Program info event!!" << endl;
		//kdDebug() << e->ev.prog_info->title << endl;
		QApplication::postEvent(_owner,
			new CCEvent(CCEvent::NewTitle, QString((char *)(e->ev.prog_info->title))));
		for (int i = 0; i < 7; i++)
			if (e->ev.prog_info->description[i][0] != 0)
				kdDebug() << i << ": "
					<< e->ev.prog_info->description[i]
					<< endl;
		break;
	default:
		assert(0);
	}
}


void CCThread::run() {
	if (!_c) {
		return;
	}

	int lines;
	double _ts = 0;
	struct timeval tv;
	tv.tv_sec = 1;
	tv.tv_usec = 0;
	int rc;
	int error = 0;

while(stopSem.available() != 0) {
	if (chanChangeSem.available() == 0) { // channel was changed
		vbi_channel_switched(_d, 0);
		chanChangeSem--;
	}

	rc = vbi_capture_read_sliced(_c, _sliced, &lines, &_ts, &tv);
	if (rc < 0) {
		kdDebug() << "VBI read error." << endl;
		if (error++ == 10)
			stopSem.tryAccess(1);
		continue;
	}

	if (rc == 0) {
		kdDebug() << "VBI timeout." << endl;
		continue;
	}

//	kdDebug() << "Read vbi data, ts=" << _ts << " lines=" << lines << " rc=" << rc << endl;
	vbi_decode(_d, _sliced, lines, _ts);
}
}


QtVisionSimon::QtVisionSimon(QtVision *qtv, QWidget *parent, const char* name)
: QVMiscPlugin(qtv, "simon-misc",parent, name), _thread(0)
{
kdDebug() << "Simon: Loading plugin." << endl;

	_cfg->setGroup("Displays");
	_cfg_displayText = _cfg->readBoolEntry("Display Text", true);
	_cfg_displayTitle = _cfg->readBoolEntry("Display Title", true);
	_cfg_pal = _cfg->readNumEntry("PAL", 0);
	_cfg_dev = _cfg->readEntry("Device", "/dev/vbi");

	connect(driver()->statusManager(), SIGNAL(channelChanged()),
		this, SLOT(channelChanged()));
	_thread = new CCThread(this, _cfg_pal, _cfg_dev);
	_thread->start();
}

QtVisionSimon::~QtVisionSimon()
{
kdDebug() << "Simon: destroying plugin." << endl;
	_thread->stop();
	_thread->wait();
	delete _thread;
}


void QtVisionSimon::channelChanged()
{
	if (_cfg_displayTitle)
		driver()->statusManager()->setProgram("");
	if (_cfg_displayText)
		driver()->osdManager()->displayCC("");
	_thread->channelChange();
}


bool QtVisionSimon::event(QEvent *e)
{
//kdDebug() << "Simon received an event." << endl;
	if (!e || e->type() != QEvent::User)
		return false;

	CCEvent *ce = static_cast<CCEvent*>(e);
	if (ce->_type == CCEvent::CaptionText) {
		if (_cfg_displayText)
			driver()->osdManager()->displayCC(ce->_text);
		return true;
	}

	if (ce->_type == CCEvent::NewTitle) {
		if (_cfg_displayTitle)
			driver()->statusManager()->setProgram(ce->_text);
		return true;
	}

return false;
}


QWidget *QtVisionSimon::configWidget(QWidget *parent, const char *name) {
QFrame *w = new QFrame(parent, name);
QGridLayout *g = new QGridLayout(w, 4, 4);

	w->setMargin(4);

	_cb_dispText = new QCheckBox(i18n("Display closed caption text"), w);
	_cb_dispTitle = new QCheckBox(i18n("Display the program title in the statusbar"), w);
	_cb_dispText->setChecked(_cfg_displayText);
	_cb_dispTitle->setChecked(_cfg_displayTitle);

	g->addMultiCellWidget(_cb_dispText, 0, 0, 0, 3);
	g->addMultiCellWidget(_cb_dispTitle, 1, 1, 0, 3);

	g->addWidget(new QLabel(i18n("Device:"), w), 2, 0);
	_cb_dev = new QComboBox(w);
	g->addMultiCellWidget(_cb_dev, 2, 2, 1, 3);

	QString dev("/dev/vbi%1");
	QStringList _devlist;

	if (!access("/dev/vbi", R_OK)) {
		_devlist << "/dev/vbi";
	}

	for (int i = 0; i <= 9; i++) {
		if (!access(dev.arg(i).local8Bit(), R_OK)) {
			_devlist << dev.arg(i);
		}
	}

	_cb_dev->insertStringList(_devlist);
	_cb_dev->setCurrentText(_cfg_dev);

	g->addWidget(new QLabel(i18n("Norm:"), w), 3, 0);
	_cb_norm = new QComboBox(w);
	g->addMultiCellWidget(_cb_norm, 3, 3, 1, 3);
	_cb_norm->insertItem("NTSC");
	_cb_norm->insertItem("PAL/SECAM");
	_cb_norm->setCurrentItem(_cfg_pal);

return w;
}


void QtVisionSimon::saveConfig() {
bool dirty = false;

	_cfg->setGroup("Displays");
	_cfg->writeEntry("Display Text", 
				(_cfg_displayText = _cb_dispText->isChecked()));
	_cfg->writeEntry("Display Title", 
			(_cfg_displayTitle = _cb_dispTitle->isChecked()));

	if (_cfg_dev != _cb_dev->currentText()) {
		dirty = true;
		_cfg->writeEntry("Device", (_cfg_dev = _cb_dev->currentText()));
	}

	if (_cfg_pal != _cb_norm->currentItem()) {
		dirty = true;
		_cfg->writeEntry("PAL", (_cfg_pal = _cb_norm->currentItem()));
	}

	_cfg->sync();

	if (dirty) { // halt the old thread
		_thread->stop();
		_thread->wait();
		delete _thread;
		_thread = 0;
	}

	if (!_cfg_displayText || dirty)
		driver()->osdManager()->displayCC("");
	if (!_cfg_displayTitle || dirty)
		driver()->statusManager()->setProgram("");

	if (dirty) { // start up a new thread
		_thread = new CCThread(this, _cfg_pal, _cfg_dev);
		_thread->start();
	}
}


extern "C" {

QtVisionSimon* create_simon(QtVision *qtv, QWidget *parent) {
QtVisionSimon *qv;

	qv = new QtVisionSimon(qtv, parent);
return qv;
}

};

#include "simon.moc"

