/*
   Copyright (c) 2002 Malte Starostik <malte@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// $Id: telex.cpp,v 1.10 2003/07/27 15:40:19 rizsanyi Exp $

#include <errno.h>

typedef unsigned char uint8_t;
#define export export_defanged
#include <libzvbi.h>
#undef export

#include <qbitmap.h>
#include <qcursor.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qthread.h>

#include <kapplication.h>
#include <kcursor.h>
#include <kdebug.h>
#include <krun.h>
#include <kaction.h>
#include <klocale.h>
#include <kxmlguifactory.h>
#include <kxmlguiclient.h>
#include <kmainwindow.h>
#include <ktoolbar.h>

#include <statusmanager.h>
#include <qtvision.h>

#include "telex.moc"

namespace Telex
{
    struct Page::Data
    {
        Data() : ref( 1 ), validity( None ), current( -1 ), request( -1 ) {}
        ~Data() { if ( validity > None ) vbi_unref_page( &page ); }
        int ref;
        Page::Validity validity;
        int current;
        int request;
        vbi_page page;
        QImage rendered;
        QImage header;
    };

    int Page::pageOffset = -1;
    Page::Page()
        : m_decoder( 0 ),
          m_number( -1 ),
          m_sub( -1 ),
          m_data( 0 )
    {
    }

    Page::Page( vbi_decoder* decoder, int number, int sub )
        : m_decoder( decoder ),
          m_number( number ),
          m_sub( sub ),
          m_data( 0 )
    {
    }

    Page& Page::operator =( const Page& rhs )
    {
        if ( rhs.m_data ) rhs.m_data->ref++;
        release();
        m_data = rhs.m_data;
        m_decoder = rhs.m_decoder;
        m_number = rhs.m_number;
        m_sub = rhs.m_sub;
        return *this;
    }

    bool Page::operator ==( const Page& rhs ) const
    {
        return *this && rhs && m_number == rhs.m_number &&
               ( m_sub == rhs.m_sub || m_sub == -1 || rhs.m_sub == -1 );
    }

    void Page::release() const
    {
        if ( m_data && !--( m_data->ref ) ) delete m_data;
    }

    bool Page::require( Validity what, int current, int request ) const
    {
        if ( !m_decoder ) return false;
        bool updated = false;
        if ( what != None && (!m_data || m_data->validity < what) )
        {
            bool navigation[] = { false, true };
            int rows[] = { 1, 25 };
            if ( !m_data ) m_data = new Data;
            else if ( m_data->validity != None )
            {
                m_data->validity = None;
                vbi_unref_page( &m_data->page );
            }
            if ( !vbi_fetch_vt_page( m_decoder,
                                     &m_data->page,
                                     vbi_dec2bcd( m_number ),
                                     m_sub >= 0 ? vbi_dec2bcd( m_sub ) : VBI_ANY_SUBNO,
                                     VBI_WST_LEVEL_3p5,
                                     rows[ what ],
                                     navigation[ what ] ) )
                return false;
            updated = true;
        }

        if ( !updated && m_data->current == current && m_data->request == request )
            return true;
        m_data->current = current;
        m_data->request = request;
        if ( pageOffset != -1 )
        {
            QString display;
            if ( request != -1 ) display = QString::number( request ) + "   ";
            else if ( current != -1 ) display = QString::number( current ) + "   ";
            if ( !display.isEmpty() )
            {
                for ( int i = pageOffset; i < pageOffset + 3; ++i )
                {
                    m_data->page.text[ i ].unicode = display[ i - pageOffset ].unicode();
                    if ( request != -1 )
                        // XXX: I want it red, and VBI_BLUE appears as red for me
                        // while VBI_RED appears as blue. Anyone else?
                        m_data->page.text[ i ].foreground = VBI_BLUE;
                }
            }
        }

        if ( m_data->rendered.width() != m_data->page.columns * 12 )
        {
            m_data->rendered = QImage( m_data->page.columns * 12, 25 * 10,
                                       32, 0, QImage::LittleEndian );
            m_data->rendered.setAlphaBuffer( true );
            m_data->header = QImage( m_data->rendered.bits(),
                                     m_data->rendered.width(), 10,
                                     32, 0, 0, QImage::LittleEndian );
        }

        vbi_draw_vt_page( &m_data->page, VBI_PIXFMT_RGBA32_LE,
                          m_data->rendered.bits(), false,  true );

        Q_UINT32* p = reinterpret_cast< Q_UINT32* >( m_data->rendered.bits() )
                      + 12 * 10 * m_data->page.columns;
        Q_UINT32* end = p + 24 * 10 * 12 * m_data->page.columns;
        if ( what == All )
            while( p < end )
            {
                if ( *p == qRgba( 0, 0, 0, 0xff ) ) *p = 0;
                ++p;
            }
        else memset( p, 0, end - p );

        m_data->validity = what;
        return true;
    }

    void Page::refresh()
    {
        if ( m_data && m_data->validity > None )
        {
            m_data->validity = None;
            vbi_unref_page( &m_data->page );
        }
    }

    QPixmap Page::header( int width, int height, int current, int request ) const
    {
        QPixmap tmp;
        if ( require( Header, current, request ) )
            tmp.convertFromImage( m_data->header.smoothScale( width, height ) );
        return tmp;
    }

    QPixmap Page::render( int width, int height, int request ) const
    {
        QPixmap tmp;
        if ( require( All, -1, request ) )
            tmp.convertFromImage( m_data->rendered.smoothScale( width, height ) );
        return tmp;
    }

    bool Page::available() const
    {
        return require( All );
    }

    int Page::columns() const
    {
        return available() ? m_data->page.columns : 0;
    }

    int Page::rows() const
    {
        return available() ? m_data->page.rows : 0;
    }

    bool Page::isLink( int col, int row ) const
    {
        return available() && m_data->page.text[ columns() * row + col ].link;
    }

    Page::Link Page::link( int col, int row ) const
    {
        if ( isLink( col, row ) )
        {
            vbi_link link;
            vbi_resolve_link( &m_data->page, col, row, &link );
            switch ( link.type )
            {
                case VBI_LINK_PAGE:
                case VBI_LINK_SUBPAGE:
                    return Page( m_decoder, vbi_bcd2dec( link.pgno ),
                                 ( link.subno == VBI_ANY_SUBNO ) ? -1 : link.subno );
                case VBI_LINK_HTTP:
                case VBI_LINK_FTP:
                case VBI_LINK_EMAIL:
                    return Page::Link( reinterpret_cast<const char *>( link.url ) );
                default: break;
            }
        }
        return Link();
    }

    struct Page::Link::Data
    {
        Data( const Page& p ) : page( p ) {}
        Data( const KURL& u ) : url( u ) {}
        Page page;
        KURL url;
    };

    Page::Link::Link( const Page& p )
        : m_type( TTX ),
          m_data( new Data( p ) )
    {
    }

    Page::Link::Link( const KURL& u )
        : m_type( URL ),
          m_data( new Data( u ) )
    {
    }

    Page::Link::~Link()
    {
        delete m_data;
    }

    const Page& Page::Link::page() const
    {
        return m_data->page;
    }

    const KURL& Page::Link::url() const
    {
        return m_data->url;
    }

    class Worker : public QThread
    {
    public:
        Worker( Plugin* parent, const QString& device );
        virtual ~Worker();

        void stop() { m_terminator.tryAccess( 1 ); }

        Page page( int page, int sub = -1 )
            { return Page( m_decoder, page, sub ); }

    protected:
        virtual void run();

    private:
        void vbiEvent( vbi_event* event );

        static void vbiEvent( vbi_event* event, void* data )
            { reinterpret_cast< Worker* >( data )->vbiEvent( event ); }

        QSemaphore m_terminator;
        Plugin* m_parent;

        vbi_capture* m_capture;
        vbi_decoder* m_decoder;
        vbi_sliced* m_sliced;
    };

    class Event : public QCustomEvent
    {
    public:
        Event( const Page& page )
            : QCustomEvent( User ), m_page( page ) {}

        const Page& page() const { return m_page; }

    private:
        Page m_page;
    };

    Worker::Worker( Plugin* parent, const QString& device )
        : QThread(),
          m_terminator( 1 ),
          m_parent( parent ),
          m_capture( 0 ),
          m_decoder( 0 ),
          m_sliced( 0 )
    {
        unsigned int services = VBI_SLICED_TELETEXT_B;
        char* cdevice = const_cast< char* >( device.latin1() );
        if (! ( m_capture = vbi_capture_v4l2_new(cdevice, 625, &services, -1, 0, 0) ) )
            m_capture = vbi_capture_v4l_new(cdevice, 625, &services, -1, 0, 0);
        if ( m_capture )
        {
            m_decoder = vbi_decoder_new();
            vbi_raw_decoder* raw = vbi_capture_parameters( m_capture );
            m_sliced = new vbi_sliced[ raw->count[ 0 ] + raw->count[ 1 ] ];
        }
    }

    Worker::~Worker()
    {
        vbi_event_handler_unregister( m_decoder, &Worker::vbiEvent, this );
        delete[] m_sliced;
        if ( m_decoder ) vbi_decoder_delete( m_decoder );
        if ( m_capture ) vbi_capture_delete( m_capture );
    }

    void Worker::vbiEvent( vbi_event* event )
    {
        if ( event->ev.ttx_page.roll_header ||
             event->ev.ttx_page.header_update ||
             event->ev.ttx_page.clock_update )
            kapp->postEvent( m_parent, new Event(
                page( vbi_bcd2dec( event->ev.ttx_page.pgno ),
                      vbi_bcd2dec( event->ev.ttx_page.subno ) ) ) );
        Page::pageOffset = event->ev.ttx_page.pn_offset;
    }

    void Worker::run()
    {
        if ( !m_capture ) return;
        vbi_event_handler_register( m_decoder, VBI_EVENT_TTX_PAGE,
                                    &Worker::vbiEvent, this );
        double ts = 0.0;
        int lines;
        timeval tv = { 1, 0 };
        while ( m_terminator.available() )
        {
            switch ( vbi_capture_read_sliced( m_capture, m_sliced, &lines, &ts, &tv ) )
            {
                case -1:
                    kdDebug() << "VBI Error: " << strerror( errno ) << endl;
                    return;
                case 0: continue;
                case 1: vbi_decode( m_decoder, m_sliced, lines, ts );
            }
        }
    }

    Plugin::Plugin( QtVision *qtv, QWidget* parent )
        : QVMiscPlugin( qtv, "telex-misc", parent ), DCOPObject("TelexIface"),
          m_worker( 0 ),
          m_pageInput( 0 )
    {
        if ( !parent ) return;

        m_display = new Display( parent );
        m_worker = new Worker( this, "/dev/vbi0" );
        m_worker->start();

        connect( m_display, SIGNAL( navigate( const Page::Link& ) ),
                 SLOT( navigate( const Page::Link& ) ) );
        connect( driver()->statusManager(), SIGNAL( channelChanged() ),
                 SLOT( channelChanged() ) );
        // disable display it will be enabled by the user
        m_display->hide();

        navigate( m_worker->page( 100 ) );
    }

    Plugin::~Plugin()
    {
        delete m_display;
        if ( m_worker )
        {
            m_worker->stop();
            m_worker->wait();
            delete m_worker;
        }
    }

    void Plugin::navigate(int page)
    {
        if (page > 100)
            navigate(m_worker->page(page));
    }

    void Plugin::nextPage()
    {
        navigate(m_page.number() + 1);
    }

    void Plugin::previousPage()
    {
        navigate(m_page.number() - 1);
    }

    void Plugin::installGUIElements(KXMLGUIFactory *guiFactory,
            KActionCollection *actionCollection)
    {
        kdDebug() << "Installing GUI Elements: " << guiFactory << " - " << actionCollection << endl;
        if (actionCollection)
        {
            KToggleAction* tta = new KToggleAction( i18n("Show Teletext"), "text_left", 0,
                    actionCollection, "toggle_teletext" );
            tta->setChecked(false);
            connect(tta, SIGNAL( toggled(bool) ), SLOT( showDisplay(bool) ));

            if (guiFactory) {
                // FIXME there must be an easier way to get the toolbar
                // get first client
                KXMLGUIClient* client = guiFactory->clients().getFirst();
                KMainWindow* mw = dynamic_cast<KMainWindow*>(client);
                if (mw)
                {
                    kdDebug() << "it's a main window" << endl;
                    QWidget* widget = mw->toolBar("viewToolBar");
                    kdDebug() << "found widget " << widget << endl;
                    if (widget)
                        tta->plug(widget);
                    else
                        kdDebug() << "No 'viewToolBar' widget" << endl;
                    // plug it to popup menu
                    //QWidget *pop = (QWidget*)guiFactory->container("screen_context_popup", mw);
                    QWidget *pop = (QWidget*)guiFactory->container("screen_context_popup", client);
                    if (pop)
                        tta->plug(pop);
                    else
                        kdDebug() << "No 'screen_context_popup' widget" << endl;
                }
            }
        }
    }

    bool Plugin::filterNumberKey( int key )
    {
        if (m_display->isHidden())
            return false;
        if ( key == -1 ) return false;
        if ( m_pageInput >= 100 ) m_pageInput = key;
        else m_pageInput = m_pageInput * 10 + key;
        if ( m_pageInput >= 100 ) navigate( m_worker->page( m_pageInput ) );
        else m_display->setRequest( m_pageInput );
        return true;
    }

    QWidget* Plugin::configWidget( QWidget* parent, const char* name )
    {
        Q_UNUSED(parent);
        Q_UNUSED(name);
        return 0;
    }

    void Plugin::saveConfig()
    {
    }

    void Plugin::channelChanged()
    {
        m_display->refresh();
    }

    void Plugin::showDisplay(bool p_show)
    {
        if (p_show)
            m_display->show();
        else
            m_display->hide();
    }

    void Plugin::toggleShow()
    {
        showDisplay(m_display->isHidden());
    }

    void Plugin::navigate( const Page::Link& link )
    {
        switch ( link.type() )
        {
            case Page::Link::TTX:
            {
                m_page = link.page();
                if ( m_page.available() ) m_display->setPage( m_page );
                else m_display->setRequest( m_page.number() );
                break;
            }
            case Page::Link::URL: new KRun( link.url() );
            default: break;
        }
    }

    void Plugin::customEvent( QCustomEvent* e )
    {
        Event* event = static_cast< Event* >( e );
        if ( event->page() == m_page ) m_display->setPage( event->page() );
        else m_display->setHeader( event->page() );
    }

    Display::Display( QWidget* parent )
        : QWidget( parent ),
          m_transparent( true ),
          m_request( -1 )
    {
        setBackgroundMode( NoBackground );
        parent->installEventFilter( this );
        parent->setMouseTracking( true );
        setMouseTracking( true );
        resize( parent->size() );
    }

    Display::~Display()
    {
        parentWidget()->setMouseTracking( false );
        parent()->removeEventFilter( this );
    }

    void Display::setHeader( const Page& page )
    {
        QPixmap header = page.header( width(), height() / 25, m_page.number(), m_request );
        bitBlt( this, QPoint( 0, 0 ), &header, header.rect(), CopyROP );
        if ( !m_scaled.isNull() )
            bitBlt( &m_scaled, QPoint( 0, 0 ), &header, header.rect(), CopyROP );
    }

    void Display::setPage( const Page& page )
    {
        m_request = -1;
        m_page = page;
        updatePage();
    }

    void Display::setRequest( int request )
    {
        m_request = request;
        updatePage();
    }

    void Display::refresh()
    {
        m_page.refresh();
        updatePage();
    }

    void Display::updatePage()
    {
        if ( m_page ) m_scaled = m_page.render( width(), height(), m_request );
        else m_scaled = QPixmap();
        if ( m_transparent ) updateMask();
        repaint();
    }

    void Display::setTransparent( bool transparent )
    {
        m_transparent = transparent;
        setAutoMask( m_transparent );
        if ( m_transparent ) updateMask();
        else clearMask();
        repaint();
    }

    void Display::updateMask()
    {
        if ( m_scaled.isNull() ) setMask( QRect( 0, 0, width(), height() / 25 ) );
        else setMask( *m_scaled.mask() );
    }

    bool Display::eventFilter( QObject*, QEvent* e )
    {
        if ( e->type() == QEvent::Resize )
        {
            resize( static_cast< QResizeEvent* >( e )->size() );
            return false;
        }
        if ( e->type() == QEvent::MouseButtonPress )
            mousePressEvent( static_cast< QMouseEvent* >( e ) );
        else if ( e->type() == QEvent::MouseButtonRelease )
            mouseReleaseEvent( static_cast< QMouseEvent* >( e ) );
        else if ( e->type() == QEvent::MouseMove )
            mouseMoveEvent( static_cast< QMouseEvent* >( e ) );
        else return false;
        return true;
    }

    void Display::resizeEvent( QResizeEvent* )
    {
        updatePage();
    }

    void Display::paintEvent( QPaintEvent* e )
    {
        if ( !m_scaled.isNull() )
            bitBlt( this, e->rect().topLeft(), &m_scaled, e->rect(), CopyROP );
    }

    void Display::mouseMoveEvent( QMouseEvent* e )
    {
        if ( !m_page.available() ) return;
        int col = m_page.columns() * e->x() / width(),
            row = m_page.rows() * e->y() / height();
        if ( m_page.available() && m_page.isLink( col, row ) )
            parentWidget()->setCursor( KCursor::handCursor() );
        else parentWidget()->setCursor( KCursor::arrowCursor() );
    }

    void Display::mousePressEvent( QMouseEvent* e )
    {
        if ( !m_page.available() ) return;
        int col = m_page.columns() * e->x() / width(),
            row = m_page.rows() * e->y() / height();
        if ( m_page.isLink( col, row ) ) emit navigate( m_page.link( col, row ) );
    }
}

extern "C"
{
    QVMiscPlugin* create_telex( QtVision *qtv, QWidget* parent )
    {
        return new Telex::Plugin( qtv, parent );
    }
}

// vim: ts=4 sw=4 et
