/*
 *
 * Copyright (C) 2002-2003 George Staikos <staikos@kde.org>
 * Copyright (C) 2002 Richard Moore <rich@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <kapplication.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "qtvision_v4l.h"

#include <qpixmap.h>
#include <qwidget.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qtimer.h>
#include <qthread.h>
#include <qevent.h>
#include <klocale.h>
#include <kdebug.h>
#include <math.h>
#include <assert.h>
#include <qapplication.h>
#include <qcolor.h>
#include <qimage.h>
#include <sys/types.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <unistd.h>
#include <stdlib.h>

#ifdef HAVE_DVBS
#include <linux/dvb/frontend.h>
#include <linux/dvb/dmx.h>
#endif

#include <kconfig.h>
#include <kscreensaver_vroot.h>

#include "v4ldev.h"
#include "v4ldevtuner.h"
#include "v4ldevcamera.h"

#include "qtvision_v4l.moc"


class KDWidget : public QWidget {
public:
	KDWidget();
	virtual ~KDWidget();
};

KDWidget::KDWidget()
: QWidget() {
	create(DefaultRootWindow(qt_xdisplay()), false, true);
}

KDWidget::~KDWidget() {
	destroy(false, false);
}


class MutexMonitor {
public:
	MutexMonitor(QMutex *m) : _m(m) { _m->lock(); }
	~MutexMonitor() { _m->unlock(); }

private:
	QMutex *_m;
};


class BLTEvent : public QEvent {
public:
	enum BLTEventType { Unknown, BLTImage };
	BLTEvent(BLTEventType t, V4LImage& i, int gsn)
		: QEvent(QEvent::User), _type(t), _image(i), _gsn(gsn)
	{ gettimeofday(&_ts, 0); }
	virtual ~BLTEvent() {}

	BLTEventType _type;
	V4LImage &_image;
	struct timeval _ts;
	int _gsn;
};


class V4LGrabber : public QThread {
public:
	V4LGrabber(QObject *owner, V4LDev *dev, QMutex& devMtx, int gsn);
	virtual ~V4LGrabber();
        virtual void run();
	void stop() { _stopSem.tryAccess(1); }

private:
	QObject *_owner;
	V4LDev *_d;
	QSemaphore _stopSem;
	QMutex& _devMtx;
	V4LImage _buf[32];  // ring buffer of images
	int _errors;
        int _gsn;
};

V4LGrabber::V4LGrabber(QObject *owner, V4LDev *dev, QMutex& devMtx, int gsn) 
: _owner(owner), _d(dev), _stopSem(1), _devMtx(devMtx), _errors(0), _gsn(gsn)
{
}

V4LGrabber::~V4LGrabber() {
}

void V4LGrabber::run() {
int bl = 0;  // buffer location
int skip = 10;   // number of frames to drop at startup (for sync purposes)
_errors = 0;
	if (!_d)
		return;

	// skip the first n frames for syncing
	while (_stopSem.available() != 0 && skip-- > 0) {
		_devMtx.lock();
		_d->grab(0, false);
		_devMtx.unlock();
	}
	
	while (_stopSem.available() != 0) {
		_devMtx.lock();
		//struct timeval tv, tv2;
		//gettimeofday(&tv, 0);
		int rc = _d->grab(&_buf[bl], false);
		/*gettimeofday(&tv2, 0);
		struct timeval tv3;
		tv3.tv_sec = tv2.tv_sec - tv.tv_sec;
		if (tv.tv_usec > tv2.tv_usec) {
			tv3.tv_sec--;
			tv3.tv_usec = tv.tv_usec-tv2.tv_usec;
		} else {
			tv3.tv_usec = tv2.tv_usec-tv.tv_usec;
		}*/
		_devMtx.unlock();
		if (rc == 0) {
/*		kdDebug() << "Post event " << (void *)&(_buf[bl]) 
			<< " time: " << tv3.tv_sec << "."
			<< tv3.tv_usec << endl; */
			QApplication::postEvent(_owner,
				new BLTEvent(BLTEvent::BLTImage,_buf[bl],_gsn));
			if (bl == 31)
				bl = 0;
			else bl++;
			_errors = 0;
		} else if (++_errors < 100) {
			continue;
		} else {
			// FIXME: send an event saying that we had 100 errors
			// and let it prompt the user to do something.  It
			// might even be good to reduce this number too.  Needs
			// investigation.
			kdDebug() << "Too many errors.  Ending V4L grabbing." << endl;
			break;
		}
	}
}


QtVisionV4L::QtVisionV4L(QtVision *qtv, QWidget *parent, const char *name)
: QVSourcePlugin(qtv, "v4l", parent, name), _w(parent), 
	_dtReg(new KDWidget), _winReg(parent), dev(0), 
	_devMtx(true), _gsn(0)
{
    g = 0;
    _probed = false;
    _capturing = false;
    _norm = 0; // auto


    _cfg->setGroup("V4L Plugin");
    _useOverlay = _cfg->readBoolEntry("Use Overlay", false); // FIXME
#ifdef HAVE_DVBS
    _dvbs = _cfg->readBoolEntry("Use DVB-S", false);
#else
    _dvbs = false;
#endif
    _qvsMethod = _cfg->readNumEntry("GD Method", QVIDEO_METHOD_XV);

    _vs = new QVideoStream(_w);
    _vs->setMethod(_qvsMethod);

    // tv window -- need visibility, structure
    XSelectInput(qt_xdisplay(), _w->winId(), ButtonPressMask      |
                                             ButtonReleaseMask    |
                                             ExposureMask         |
                                             PointerMotionMask    |
                                             VisibilityChangeMask |
                                             StructureNotifyMask  |
                                             SubstructureNotifyMask);

    // root window -- need visibility, sub-, structure
    XSelectInput(qt_xdisplay(), DefaultRootWindow(qt_xdisplay()),
                                       VisibilityChangeMask |
                                       FocusChangeMask      |
                                       StructureNotifyMask  |
                                       SubstructureNotifyMask);

    kdDebug() << "QtVision V4L plugin loaded successfully." << endl;

}


QtVisionV4L::~QtVisionV4L()
{
    stopVideo();
    delete dev;
    dev = 0;
    delete _vs;
    delete _dtReg;
}


bool QtVisionV4L::isTuner()
{
MutexMonitor l(&_devMtx);
    return dev && dev->isTuner();
}


int QtVisionV4L::brightness()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->brightness() : -1;
}


QColor QtVisionV4L::colourKey()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->colourKey() : QColor();
}


int QtVisionV4L::colour()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->colour() : -1;
}


int QtVisionV4L::hue()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->hue() : -1;
}


int QtVisionV4L::contrast()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->contrast() : -1;
}


int QtVisionV4L::whiteness()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->contrast() : -1;
}


int QtVisionV4L::signal()
{
MutexMonitor l(&_devMtx);
    return dev ? dev->signal() : -1;
}


/* ARGH, this doesn't work on my card.  It always returns 0! */
bool QtVisionV4L::muted()
{
MutexMonitor l(&_devMtx);
    if (dev)
        return dev->audioEnabled();
    return false;
}


int QtVisionV4L::tunerMode()
{
MutexMonitor l(&_devMtx);
    if ( dev && dev->isTuner() ) {
	V4LTuner *tuner = static_cast<V4LTuner *>( dev );
	return tuner->tunerMode();
    }
    return -1; // unsupported
}


int QtVisionV4L::setDevice( const QString &name )
{
    assert(!dev);
    if (!_probed)
        probeDevices();
    _device = name;
    _currentDev = _devNames[name];
    kdDebug() << "V4L: setDevice [" << name 
		<< "] which maps to " << _currentDev << endl;
    int rc = v4lSetup(); // FIXME: check rc here
    dev = V4LDev::getDevice( _currentDev );
    kdDebug() << "V4L: Success? " << (dev ? "true" : "false") << endl;

    return dev ? 0 : -1;
}


int QtVisionV4L::setSource( const QString &src )
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return -1;
    
    _source = src;
    int rc=dev->setSource( src );
    setNorm(_norm);
    return rc;
}


void QtVisionV4L::setNorm( int n )
{
MutexMonitor l(&_devMtx);
    if ( dev && dev->isTuner() ) {
	V4LTuner *tuner = static_cast<V4LTuner*>( dev );
	tuner->setTunerMode( n );
        _norm = n;
    }
}


int QtVisionV4L::norm() const
{
    return _norm;
}


int QtVisionV4L::frequency()
{
MutexMonitor l(&_devMtx);
    if (dev && dev->isTuner())
        return 125 * static_cast<V4LTuner*>(dev)->freq() / 2;
    return -1;
}


void QtVisionV4L::setFrequency( int freq )
{
MutexMonitor l(&_devMtx);
    if ( !dev || !dev->isTuner() )
	return;
    static_cast<V4LTuner*>(dev)->setFreq( freq * 2 / 125);
}


void QtVisionV4L::setMuted( bool muted )
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;

    if ( muted )
	dev->disableAudio();
    else
	dev->enableAudio();
}


void QtVisionV4L::setBrightness(int val)
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;
    dev->setBrightness( val );
}


void QtVisionV4L::setColour(int val)
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;
    dev->setColour( val );
}


void QtVisionV4L::setHue(int val)
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;
    dev->setHue( val );
}


void QtVisionV4L::setContrast(int val) {
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;
    dev->setContrast( val );
}


void QtVisionV4L::setWhiteness(int val)
{
MutexMonitor l(&_devMtx);
    if ( !dev )
	return;
    dev->setWhiteness( val );
}


void QtVisionV4L::viewMoved(int x, int y)
{
MutexMonitor l(&_devMtx);
//kdDebug() << "View moved!" << endl;
    if (dev->overlayOn()) {
        int scn = QApplication::desktop()->screenNumber(_w->mapToGlobal(_w->geometry().center()));
        QRect g;
        g.moveTopLeft(_w->mapToGlobal(QPoint(x,y))-QApplication::desktop()->screenGeometry(scn).topLeft());
        g.setSize(_w->size());
        dev->setCaptureGeometry(g);
        _doRefresh = true;
        _refresh_ul = QApplication::desktop()->screenGeometry(scn).topLeft();
        _refresh_sz = QApplication::desktop()->screenGeometry(scn).size();
        QTimer::singleShot(0, this, SLOT(triggerRepaint()));
    }
}


void QtVisionV4L::viewResized(int w, int h)
{
MutexMonitor l(&_devMtx);
    if (dev)
        dev->setImageSize(w, h);

    kdDebug() << "viewResized " << w << " " << h << endl;

    _vs->setSize(QSize(w, h));
}
    

int QtVisionV4L::probeDevices()
{
	QString dev;
	struct stat sb;

	int rc = stat("/dev/v4l", &sb);
	if (!rc && S_ISDIR(sb.st_mode) && !access("/dev/v4l", R_OK|X_OK)) {
		// DEVFS
		dev = "/dev/v4l/video%1";
	} else {
		// Normal V4L case
		dev = "/dev/video%1";
	}

	_devices.clear();
	_sources.clear();
	_tuners.clear();
	_devNames.clear();

	// FIXME: when we test for devices, we must make sure that we can
	// actually use them too.  This fails for cases where something else
	// is using the device currently.
	if (!access("/dev/video", R_OK|W_OK)) {
		V4LDev *vd = V4LDev::getDevice("/dev/video");
		if (vd) {
			QString name = vd->name() + " Video4Linux(/dev/video)";
			_tuners[name] = vd->isTuner();
			_sources[name] = vd->sources();
			_devices << name;
			_devNames[name] = "/dev/video";
			delete vd;
		}
	}

	for (int i = 0; i <= 9; i++) {
		QString dn = dev.arg(i);
		if (!access(dn.local8Bit(), R_OK|W_OK)) {
			V4LDev *vd = V4LDev::getDevice(dn);
			if (vd) {
				QString name =vd->name()+" Video4Linux("+dn+")";
				_tuners[name] = vd->isTuner();
				_sources[name] = vd->sources();
				_devices << name;
				_devNames[name] = dn;
				delete vd;
			}
		}
	}

    _probed = true;
    return 0;
}


int QtVisionV4L::v4lSetup()
{
    QString kv4lcall("kv4lsetup -q -c ");
    kv4lcall += _currentDev;
    //kv4lcall += QString(" -l -b ") + QString::number(QPaintDevice::x11AppDepth());
    // FIXME: configure shift
    // kv4lcall += QString(" -s ") + QString::number(shift);

    // FIXME: use kprocess instead
    kdDebug() << "kv4lsetup execution line: " << kv4lcall << endl;

    switch (system(kv4lcall.latin1())) {
    case -1:
        kdWarning() << "kwintv: Couldn't start kv4lsetup!" << endl;
     return -1;
    case 0:
     return 0;
    default:
        kdWarning() << "kwintv: kv4lsetup had some trouble.  Trying to continue anyway." << endl;
     return -1;
    }
}

int QtVisionV4L::startVideo()
{
int rc = 0;
    if (!dev || g || _capturing) {
	kdDebug() << "Error starting video: " << (void *)dev << " " << (void*)g << endl;
        return -1;
    }

    // FIXME - does this break aspect ratio?  probably
    dev->setImageSize(_w->width(), _w->height());
    if (_useOverlay && dev->canOverlay()) {
        QPoint qp = _w->mapToGlobal(_w->pos());
        int sn = QApplication::desktop()->screenNumber(_w);
        qp -= QApplication::desktop()->screenGeometry(sn).topLeft();
        dev->setColourKey(0x0000ff00); // FIXME: we need to do proper colorkey for devices that support it.
	KApplication::kApplication()->installX11EventFilter(_clipper = new Clipper);
        _clipper->setWidget(_w);
        connect(_clipper, SIGNAL(updateClipping()), this, SLOT(updateClipping()));
        rc = dev->startCapture(qp.x(), qp.y());
    } else {
        g = new V4LGrabber(this, dev, _devMtx, ++_gsn);
        g->start();
        _vs->setSize(_w->size());
        _doRefresh = true;
        int scn = QApplication::desktop()->screenNumber(_w->mapToGlobal(_w->geometry().center()));
        _refresh_ul = QApplication::desktop()->screenGeometry(scn).topLeft();
        _refresh_sz = QApplication::desktop()->screenGeometry(scn).size();
        triggerRepaint();
    }

    if (rc == 0)
        setMuted(false);

    _capturing = true;
return rc;
}


int QtVisionV4L::stopVideo()
{
    if (!_capturing)
        return -1;

    setMuted(true);

    if (g) {
        g->stop();
        g->wait();
	V4LGrabber *t = g;
	g = 0;
        delete t;
	_w->erase();
    } else {  // overlay
        dev->stopCapture();
        delete _clipper;
        _doRefresh = true;
        int scn = QApplication::desktop()->screenNumber(_w->mapToGlobal(_w->geometry().center()));
        _refresh_ul = QApplication::desktop()->screenGeometry(scn).topLeft();
        _refresh_sz = QApplication::desktop()->screenGeometry(scn).size();
        triggerRepaint();
    }
    _capturing = false;
    return 0;
}


bool QtVisionV4L::canVideoDesktop() const { return true; }


int QtVisionV4L::setVideoDesktop(bool on) {
    if (!dev)
        return -1;

    MutexMonitor l(&_devMtx);
    if (on) {
        _wReg = _vs->width();
        _hReg = _vs->height();
        stopVideo();
        _w = _dtReg;
        delete _vs;
        _vs = new QVideoStream(_w);
        _vs->setMethod(QVIDEO_METHOD_XV);
        viewResized(_w->width(), _w->height());
        _isVideoDesktop = true;
        startVideo();
        setMuted(false);
        _capturing = true;
    } else {
        if (!_isVideoDesktop)
            return -1;
        _isVideoDesktop = false;
        stopVideo();
        setMuted(true);
	_dtReg->repaint();
        _w = _winReg;
        delete _vs;
        _vs = new QVideoStream(_w);
        _vs->setMethod(QVIDEO_METHOD_XV);
        viewResized(_wReg, _hReg);
        return startVideo();
    }

    return 0;
}


bool QtVisionV4L::videoPlaying() const {
    return _capturing;
}

bool QtVisionV4L::canGrabStill() const
{
    // Is this dangerous?  We can't use a mutex here though...
    return dev && dev->canGrab();
}

bool QtVisionV4L::grabStill( QPixmap *pix )
{
MutexMonitor l(&_devMtx);
    if (dev && dev->canGrab()) {
        V4LImage im;
        int fmt = dev->inputFormat();
        dev->setInputFormat(VIDEO_PALETTE_RGB24);
        int rc = 1, timeout = 20;
        while (rc != 0 && timeout-- > 0) {
            rc = dev->grab(&im, false);
        }
        rc = dev->grab(&im, false);
        dev->setInputFormat(fmt);
        if (rc == 0) {
            uchar *buf = new uchar[im.width*im.height*4];
            QImage i(buf, im.width, im.height, 32, 0, 0, QImage::LittleEndian);
            rc = im.toQImage(i);
            if (rc == 0) {
                pix->convertFromImage(i);
                delete[] buf;
                return true;
            }
            delete[] buf;
        }
    }
    return false;
}


void QtVisionV4L::updateClipping() {
Display *dpy = qt_xdisplay();
Window win = _w->winId();
Window root;
Window rroot, parent, *children;
unsigned int nchildren = 0;
unsigned int i = 0;

    dev->clearClips();
//printf("WINDOW: %ld / %ld\n", _w->winId(), _w->parentWidget()->winId());

    int scn = QApplication::desktop()->screenNumber(_w->mapToGlobal(_w->geometry().center()));
    if (QApplication::desktop()->isVirtualDesktop()) {
        root = XRootWindow(dpy, scn);
    } else {
        root = DefaultRootWindow(dpy);
    }

    // Find the window just below the root.
    for (;;) {
        if (!XQueryTree(dpy, win, &rroot, &parent, &children, &nchildren))
            return;
        XFree(children);
        if (root == parent)
            break;
        win = parent;
    }

    if (!XQueryTree(dpy, root, &rroot, &parent, &children, &nchildren)) {
        return;
    }

    // Find the root children that come after the window we just found
    for  (i = 0; i < nchildren; i++ ) {
        if ( children[i] == win ) {
            break;
        }
    }

    QRect wgeom = _w->geometry();
    QRect geom;
    QPoint ul = _w->mapToGlobal(wgeom.topLeft());
    QPoint lr = _w->mapToGlobal(wgeom.bottomRight());

    // Go through the rest and see if they overlap.  If so, clip.
    for (i++; i < nchildren; i++) {
        XWindowAttributes wattrs;

        XGetWindowAttributes(dpy, children[i], &wattrs);
        if (!(wattrs.map_state & IsViewable))
            continue;

        if (wattrs.x + wattrs.width < ul.x() || wattrs.x > lr.x() ||
            wattrs.y + wattrs.height < ul.y() || wattrs.y > lr.y())
            continue;

        scn = QApplication::desktop()->screenNumber(QRect(wattrs.x, wattrs.y, wattrs.width, wattrs.height).center());
	geom = QApplication::desktop()->screenGeometry(scn);
        wattrs.x -= geom.x();
        wattrs.y -= geom.y();
//printf("Found a sibling of the view: %d %d %dx%d\n", wattrs.x, wattrs.y, wattrs.width, wattrs.height);

        dev->addClip(QRect(wattrs.x, wattrs.y, wattrs.width, wattrs.height));
    }

    XFree(children);

    if (XQueryTree(dpy, _w->winId(), &rroot, &parent, &children, &nchildren)) {
        // Find the root children that come after the window we just found
        for  (i = 0; i < nchildren; i++ ) {
            XWindowAttributes wattrs;

            XGetWindowAttributes(dpy, children[i], &wattrs);
            if (!(wattrs.map_state & IsViewable))
                continue;

            QPoint wpt(wattrs.x, wattrs.y);
            wpt = _w->mapToGlobal(wpt);
            wattrs.x = wpt.x();
            wattrs.y = wpt.y();
            if (wattrs.x + wattrs.width < ul.x() || wattrs.x > lr.x() ||
                wattrs.y + wattrs.height < ul.y() || wattrs.y > lr.y())
                continue;

            scn = QApplication::desktop()->screenNumber(QRect(wattrs.x, wattrs.y, wattrs.width, wattrs.height).center());
            geom = QApplication::desktop()->screenGeometry(scn);
            wattrs.x -= geom.x();
            wattrs.y -= geom.y();
//printf("Found a child of the view: %d %d %dx%d\n", wattrs.x, wattrs.y, wattrs.width, wattrs.height);

            dev->addClip(QRect(wattrs.x, wattrs.y, wattrs.width, wattrs.height));
        }
        XFree(children);
    }

    dev->reClip();

    _doRefresh = true;
    _refresh_ul = QApplication::desktop()->screenGeometry(scn).topLeft();
    _refresh_sz = QApplication::desktop()->screenGeometry(scn).size();
    QTimer::singleShot(0, this, SLOT(triggerRepaint()));

}

void QtVisionV4L::triggerRepaint() {
    if (_doRefresh) {
        repaintRegion(_refresh_ul, _refresh_sz);
        _doRefresh = false;
    }
}


void QtVisionV4L::repaintRegion(QPoint ul, QSize sz) {
    // repaint the window to clean it up
    XSetWindowAttributes xwattr;
    xwattr.override_redirect = true;
    xwattr.backing_store = NotUseful;
    xwattr.save_under = false;
    unsigned long mask = CWSaveUnder | CWBackingStore | CWOverrideRedirect;

    Display *dpy = qt_xdisplay();
    Window root;
    int scn = QApplication::desktop()->screenNumber(QRect(ul, sz).center());
    if (QApplication::desktop()->isVirtualDesktop()) {
        root = XRootWindow(dpy, scn);
    } else {
        root = DefaultRootWindow(dpy);
    }

    Window pwin = XCreateWindow(dpy, root,
                                ul.x(), ul.y(), sz.width(), sz.height(), 0,
                                CopyFromParent, InputOutput, CopyFromParent,
                                mask, &xwattr);
    XMapWindow(dpy, pwin);
    XUnmapWindow(dpy, pwin);
    XDestroyWindow(dpy, pwin);
    QApplication::flushX();
}


bool QtVisionV4L::event(QEvent *e)
{
//kdDebug() << "V4L plugin received an event." << endl;
    if (!g || !e || e->type() != QEvent::User)
        return false;

    BLTEvent *be = static_cast<BLTEvent*>(e);
    struct timeval ts;
    gettimeofday(&ts, 0);
    if (be->_type == BLTEvent::BLTImage &&
        be->_gsn == _gsn &&
	be->_ts.tv_sec == ts.tv_sec &&
	ts.tv_usec - be->_ts.tv_usec < 200000 ) {
        _vs->setInputSize(QSize(be->_image.width, be->_image.height));
        *_vs << be->_image.buffer;
        return true;
    }

return false;
}


bool QtVisionV4L::needsNorm() const
{
return true;
}


Clipper::Clipper()
: QWidget(0, 0), _w(0) {
}


Clipper::~Clipper() {
}


void Clipper::setWidget(QWidget *w) {
    _w = w;
}


bool Clipper::x11Event(XEvent *e) {
    assert(e);

    switch (e->type) {
    case GraphicsExpose:
        kdDebug() << "Got a graphics expose event" << endl;
        emit updateClipping();
    break;
    case VisibilityNotify:
        switch (e->xvisibility.state) {
        case VisibilityFullyObscured:
//            kdDebug() << "Got a visibility obscured event" << endl;
//            emit updateClipping();
        break;
        case VisibilityPartiallyObscured:
//            kdDebug() << "Got a visibility event" << endl;
//            emit updateClipping();
        break;
        case VisibilityUnobscured:
            kdDebug() << "Got a visibility unobscured event" << endl;
            emit updateClipping();
        break;
        }
    break;
    /* Keep me - this happens on resizes */
    case ConfigureNotify:
        if (e->xconfigure.window == _w->winId()) {
            kdDebug() << "Got a configure notify event" << endl;
            emit updateClipping();
        }
    break;
    case PropertyNotify:
        if (e->xproperty.window == _w->winId()) {
            kdDebug() << "Got a property notify event" << endl;
            emit updateClipping();
        }
    break;
    case MapNotify:
        kdDebug() << "Got a map event" << endl;
        emit updateClipping();
    break;
    case ReparentNotify:
        kdDebug() << "Got a reparent notify event" << endl;
        emit updateClipping();
    break;
    case UnmapNotify:
        kdDebug() << "Got an unmap notify event" << endl;
        emit updateClipping();
    break;
    case ClientMessage:
    case DestroyNotify:
    case Expose:
    case FocusOut:
    case FocusIn:
    case EnterNotify:
    case LeaveNotify:
    case KeymapNotify:
    case ButtonPress:
    case ButtonRelease:
    case MotionNotify:
    break;
    default:
        kdDebug() << "Got an event " << e->type << endl;
    break;
    }

return false;
}


QWidget *QtVisionV4L::configWidget(QWidget *parent, const char *name) {
    _cfgWidget = new V4LPluginCfg(parent, name);

    _cfgWidget->_xvideo->setEnabled(_vs->haveMethod(QVIDEO_METHOD_XV));
    _cfgWidget->_xvshm->setEnabled(_vs->haveMethod(QVIDEO_METHOD_XVSHM));
    _cfgWidget->_x11->setEnabled(_vs->haveMethod(QVIDEO_METHOD_X11));
    _cfgWidget->_x11shm->setEnabled(_vs->haveMethod(QVIDEO_METHOD_XSHM));

    switch(_qvsMethod) {
    case QVIDEO_METHOD_XV:
        _cfgWidget->_xvideo->setChecked(true);
    break;
    case QVIDEO_METHOD_XVSHM:
        _cfgWidget->_xvshm->setChecked(true);
    break;
    case QVIDEO_METHOD_X11:
        _cfgWidget->_x11->setChecked(true);
    break;
    case QVIDEO_METHOD_XSHM:
        _cfgWidget->_x11shm->setChecked(true);
    break;
    default:
        assert(0);
    break;
    }

    _cfgWidget->_overlay->setChecked(_useOverlay);
#ifdef HAVE_DVBS
    _cfgWidget->_dvbs->setChecked(_dvbs);
#else
    _cfgWidget->_dvbs->setChecked(false);
    _cfgWidget->_dvbs->setEnabled(false);
#endif

return _cfgWidget;
}


void QtVisionV4L::saveConfig() {
    _useOverlay = _cfgWidget->_overlay->isChecked();
    _dvbs = _cfgWidget->_dvbs->isChecked();

    _qvsMethod = QVIDEO_METHOD_NONE;
    if (_cfgWidget->_xvideo->isChecked()) {
        _qvsMethod = QVIDEO_METHOD_XV;
    } else if (_cfgWidget->_xvshm->isChecked()) {
        _qvsMethod = QVIDEO_METHOD_XVSHM;
    } else if (_cfgWidget->_x11shm->isChecked()) {
        _qvsMethod = QVIDEO_METHOD_XSHM;
    } else if (_cfgWidget->_x11->isChecked()) {
        _qvsMethod = QVIDEO_METHOD_X11;
    }

    _cfg->writeEntry("Use Overlay", _useOverlay);
    _cfg->writeEntry("GD Method", _qvsMethod);
    _cfg->writeEntry("Use DVB-S", _dvbs);

    if (_capturing) {
        stopVideo();
        _vs->setMethod(_qvsMethod);
        startVideo();
    } else {
        _vs->setMethod(_qvsMethod);
    }

    _cfg->sync();
}


extern "C" {

    QtVisionV4L* create_v4l(QtVision *qtv, QWidget *w) {
	return new QtVisionV4L(qtv, w, "v4l plugin");
    }

};




