/*
 *
 * Copyright (C) 2002 George Staikos <staikos@kde.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <qpixmap.h>
#include <qwidget.h>
#include <qtimer.h>
#include <klocale.h>
#include "qtvision_xv.h"
#include <kdebug.h>
#include <math.h>
#include <assert.h>
#include <qapplication.h>

#include <kscreensaver_vroot.h>


QtVisionXv::QtVisionXv(QtVision *qtv, QWidget *parent, const char* name)
: QVSourcePlugin(qtv, "xv",parent,name) {
xvDevice = NULL;
xvHandle = NULL;
kdDebug() << "QtVision XVideo plugin loaded successfully." << endl;

}


QtVisionXv::~QtVisionXv() {
stopVideo();
delete xvHandle;
kdDebug() << "QtVision XVideo plugin unloaded." << endl;
}


bool QtVisionXv::isTuner() {
	if (xvDevice) {
		return xvDevice->getAttribute("XV_FREQ", NULL);
	}
return false;
}


#define my_round(x) ((x)-int(x) > 0.50 ? int(x)+1 : int(x))

int QtVisionXv::brightness() {
	if (xvDevice) {
		int val, minv, maxv;
		xvDevice->getAttributeRange("XV_BRIGHTNESS", &minv, &maxv);
		return 65535
		* (xvDevice->getAttribute("XV_BRIGHTNESS", &val) ? val-minv : 0)
		/ (maxv - minv);
	}

return -1;
}


QColor QtVisionXv::colourKey() {
	if (xvDevice) {
		int val = 0;
		xvDevice->getAttribute("XV_COLORKEY", &val);
		XColor a;
		memset(&a, 0, sizeof(a));
		a.pixel = val;
		XQueryColor(qt_xdisplay(), 
#if QT_VERSION >= 0x030100
			    QPaintDevice::x11AppColormap( qt_xscreen() ),
#else
			    QPaintDevice::x11AppColormap(),
#endif
			    &a);
		return QColor(QColor(a.red, a.green, a.blue).rgb(), val);
	}

return QColor();
}


int QtVisionXv::colour() {
	if (xvDevice) {
		int val, minv, maxv;
		xvDevice->getAttributeRange("XV_SATURATION", &minv, &maxv);
		return 65535
		* (xvDevice->getAttribute("XV_SATURATION", &val) ? val-minv : 0)
		/ (maxv - minv);
	}

return -1;
}


int QtVisionXv::hue() {
	if (xvDevice) {
		int val, minv, maxv;
		xvDevice->getAttributeRange("XV_HUE", &minv, &maxv);
		return 65535
		* (xvDevice->getAttribute("XV_HUE", &val) ? val-minv : 0)
		/ (maxv - minv);
	}

return -1;
}


int QtVisionXv::contrast() {
	if (xvDevice) {
		int val, minv, maxv;
		xvDevice->getAttributeRange("XV_CONTRAST", &minv, &maxv);
		return 65535
		* (xvDevice->getAttribute("XV_CONTRAST", &val) ? val-minv : 0)
		/ (maxv - minv);
	}

return -1;
}


int QtVisionXv::whiteness() {
return 0; // unsupported
}


int QtVisionXv::signal() {
	if (xvDevice) {
		int v;
		if (xvDevice->getAttribute("XV_SIGNAL", &v))
			return v;
	}

return -1;
}


int QtVisionXv::frequency() {
	if (xvDevice) {
		int v;
		if (xvDevice->getAttribute("XV_FREQ", &v))
			return v*125/2;
	}

return -1;
}


/* ARGH, this doesn't work on my card.  It always returns 0! */
bool QtVisionXv::muted() {
	if (xvDevice) {
		int v;
		if (xvDevice->getAttribute("XV_MUTE", &v)) {
			kdDebug() << "XV_MUTE == " << v << endl;
			return (v==1);
		}
	}

return false;
}


int QtVisionXv::tunerMode() {
	return -1; // unsupported
}


int QtVisionXv::setDevice( const QString& dev ) {
	KXvDeviceList& xvdl(xvHandle->devices());
	KXvDevice *xvdev = NULL;

	for (xvdev = xvdl.first(); xvdev; xvdev = xvdl.next()) {
		QString label = i18n("%1 - XVideo port %2").arg(xvdev->name()).arg(xvdev->port());
		if (dev == label) {
			stopVideo();
			xvDevice = xvdev;
			_device = dev;
			_source = "";
			return 0;
		}
	}

return -1;
}


int QtVisionXv::setSource( const QString &src ) {
	if (_device.isEmpty() || !xvDevice)
		return -1;

	if (xvDevice->encodings().contains(src)) {
		_source = src;
		xvDevice->setEncoding(src);
		return 0;
	}

return -2;
}


void QtVisionXv::setTunerMode( int ) {
	// This is not used here
}


void QtVisionXv::setFrequency( int freq ) {
	if (xvDevice) {
		xvDevice->setAttribute("XV_FREQ", freq*2/125);
		// I hate this hack.  How do we do this right?
	}
}


void QtVisionXv::setMuted( bool muted ) {
	if (xvDevice) {
		xvDevice->setAttribute("XV_MUTE", muted ? 1 : 0);
	}
}


#define SCALE_FACTOR float factor = float(val)*float(maxv-minv)/float(65535) + float(minv)

void QtVisionXv::setBrightness(int val) {
	if (xvDevice) {
		int minv, maxv;
		xvDevice->getAttributeRange("XV_BRIGHTNESS", &minv, &maxv);
		SCALE_FACTOR;
		xvDevice->setAttribute("XV_BRIGHTNESS", my_round(factor));
	}
}


void QtVisionXv::setColour(int val) {
	if (xvDevice) {
		int minv, maxv;
		xvDevice->getAttributeRange("XV_SATURATION", &minv, &maxv);
		SCALE_FACTOR;
		xvDevice->setAttribute("XV_SATURATION", my_round(factor));
	}
}


void QtVisionXv::setHue(int val) {
	if (xvDevice) {
		int minv, maxv;
		xvDevice->getAttributeRange("XV_HUE", &minv, &maxv);
		SCALE_FACTOR;
		xvDevice->setAttribute("XV_HUE", my_round(factor));
	}
}


void QtVisionXv::setContrast(int val) {
	if (xvDevice) {
		int minv, maxv;
		xvDevice->getAttributeRange("XV_CONTRAST", &minv, &maxv);
		SCALE_FACTOR;
		xvDevice->setAttribute("XV_CONTRAST", my_round(factor));
	}
}

#undef SCALE_FACTOR

void QtVisionXv::setWhiteness(int) {
//unsupported
}


void QtVisionXv::viewResized(int /*w*/, int /*h*/) {
	QTimer::singleShot(0, this, SLOT(startVideo()));
}
    

int QtVisionXv::probeDevices() {
	KXvDeviceList& xvdl(xvHandle->devices());
	KXvDevice *xvdev = NULL;

	_devices.clear();
	_sources.clear();
	_tuners.clear();

	for (xvdev = xvdl.first(); xvdev; xvdev = xvdl.next()) {
		if (xvdev->isVideoSource() && xvdev->supportsWidget(_widget)) {
			QString label = i18n("%1 - XVideo port %2").arg(xvdev->name()).arg(xvdev->port());
			_devices << label;
			_sources[label] = xvdev->encodings();
			_tuners[label] = xvdev->getAttribute("XV_FREQ", NULL);
		}
	}
return 0;
}


int QtVisionXv::startVideo() {
	assert(_widget);
	if (!xvDevice || _isVideoDesktop)
		return -1;
	bool rc = xvDevice->startVideo(_widget, _widget->width(), _widget->height());
	if (!rc)
		kdDebug() << "Error starting video in Xv plugin!" << endl;
	setMuted(true);
	setMuted(false);
return rc ? 0 : -2;
}


int QtVisionXv::stopVideo() {
	setMuted(true);
	if (!xvDevice || _isVideoDesktop)
		return -1;
	return xvDevice->stopVideo() ? 0 : -1;
}


bool QtVisionXv::canVideoDesktop() const { return true; }


int QtVisionXv::setVideoDesktop(bool on) {
	if (!xvDevice)
		return -1;

	if (on) {
		Window w = DefaultRootWindow(qt_xdisplay());
		const QRect& s = QApplication::desktop()->screenGeometry();
		stopVideo();
		bool rc = xvDevice->startVideo(w, s.width(), s.height());
		setMuted(false);
		_isVideoDesktop = true;
	} else {
		if (!_isVideoDesktop)
			return -1;
		_isVideoDesktop = false;
		xvDevice->stopVideo();
		setMuted(true);
		Window w = DefaultRootWindow(qt_xdisplay());
		// FIXME: force a repaint of w.
		return startVideo();
	}
return 0;
}


bool QtVisionXv::videoPlaying() const {
	if (xvDevice)
		return xvDevice->videoPlaying();
	return false;
}

bool QtVisionXv::canGrabStill() const
{
    return true;
}

bool QtVisionXv::grabStill( QPixmap *pix )
{
    if (pix && xvDevice)
	return xvDevice->grabStill( pix, pix->width(), pix->height() );
    return false;
}


extern "C" {

QtVisionXv* create_xv(QtVision *qtv, QWidget *w) {
Drawable d;
KXv *xv;
QtVisionXv *qv;

	if (!w || !KXv::haveXv())
		return NULL;

	d = w->winId();

	xv = KXv::connect(d);
	if (!xv)
		return NULL;

	qv = new QtVisionXv(qtv, w);
	qv->xvHandle = xv;
	qv->setWidget(w);

return qv;
}

};

#include "qtvision_xv.moc"

