#include <ctype.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qtextbrowser.h>
#include <qtimer.h>

#include <kdebug.h>

#include "teletextsource.h"

#include "teletextwidget.h"
#include "teletextwidget.moc"

TeletextWidget::TeletextWidget( QWidget *parent, const char *name, WFlags wflags )
    : QVBox( parent, name, wflags )
{ 
    setFrameStyle( QFrame::Panel | QFrame::Sunken );
    setLineWidth( 2 );

    header = new QLabel( this );
    header->setBackgroundColor( Qt::blue );
    header->setPaletteForegroundColor( Qt::white );

    view = new QTextBrowser( this );
    view->setFrameStyle( QFrame::NoFrame );
    view->setText("<qt bgcolor=\"black\" text=\"white\" link=\"blue\"></qt>");

    src = new TeletextSource( this, "teletext_source" );
    connect( src, SIGNAL( headerChanged(const char *) ), this, SLOT( setHeader(const char *) ) );
    connect( src, SIGNAL( pageChanged(TeletextPage *) ), this, SLOT( setPage(TeletextPage *) ) );

    QTimer *t = new QTimer( this );
    connect( t, SIGNAL(timeout()), this, SLOT(updateHeader()) );
    t->start(500);
}

TeletextWidget::~TeletextWidget()
{
}

void TeletextWidget::setPage( TeletextPage *page )
{
    setCaption( QString("Page: %1").arg(page->pgno, 0, 16) );

    QString text;

    text.append("<qt bgcolor=\"black\" text=\"white\" link=\"blue\"><pre>");
    for ( int line = 1 ; line < 25 ; line++ ) {
	const char *data = (const char *) page->data[line];

	text.append( createLinks(parseLine(data)) );
	text.append( "\n" );
    }
    text.append("</pre></qt>");

    view->setText( text );
}

void TeletextWidget::updateHeader()
{
    header->setText( parseHeader( hdr.data() ) );
}

void TeletextWidget::setHeader( const char *header )
{
    hdr = header;
}

QString TeletextWidget::parseHeader( const char *hdr )
{
    if (!hdr)
        return QString();

    QString s(" %1    %2");
    return s.arg( QString::number(source()->pageNumber(), 16) ).arg( parseLine(hdr+8) );
}

QString TeletextWidget::parseLine( const char *data )
{
    if (!data)
        return QString();

    QCString buff( data, 41 );
    QString line;
    bool inFont = false;

    for ( uint i = 0 ; i < buff.length() ; i++ ) {
	unsigned char c = data[i];

	if ( c == 0 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"black\">" );
	    inFont = true;
	}
	else if ( c == 1 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"red\">" );
	    inFont = true;
	}
	else if ( c == 2 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"green\">" );
	    inFont = true;
	}
	else if ( c == 3 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"yellow\">" );
	    inFont = true;
	}
	else if ( c == 4 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"blue\">" );
	    inFont = true;
	}
	else if ( c == 5 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"magenta\">" );
	    inFont = true;
	}
	else if ( c == 6 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"cyan\">" );
	    inFont = true;
	}
	else if ( c == 7 ) {
	    if ( inFont )
		line.append( "</font>" );
	    line.append( " <font color=\"white\">" );
	    inFont = true;
	}
	else if ( c == 8 ) {
	    // Flash
	    line.append( '@' );
	}
	else if ( c == 9 ) {
	    // Steady
	    line.append( '@' );
	}
	else if ( c == 10 ) {
	    // Normal height
	    line.append( '@' );
	}
	else if ( c == 11 ) {
	    // Double height
	    line.append( '@' );
	}
	else if ( (c > 16) && (c < 24) ) {
	    // Start gfx
	    line.append('@');
	}
	else if ( c == 25 ) {
	    // Continuous gfx
	    line.append( '@' );
	}
	else if ( c == 26 ) {
	    // Separated gfx
	    line.append( '@' );
	}
	else if ( c == 28 ) {
	    // Black bg
	    line.append( '@' );
	}
	else if ( c == 29 ) {
	    // New bg
	    line.append( '@' );
	}
	else if ( c == 30 ) {
	    // Hold gfx
	    line.append( '@' );
	}
	else if ( c == 31 ) {
	    // Release gfx
	    line.append( '@' );
	}
	else if ( !isprint(c) ) {
	    line.append( ' ' );
	}
	else if ( c == '&' ) {
	    line.append( "&amp;" );
	}
	else if ( c == '"' ) {
	    line.append( "&quot;" );
	}
	else if ( c == '<' ) {
	    line.append( "&lt;" );
	}
	else {
	    line.append( buff[i] );
	}
    }

    if ( inFont )
	line.append( "</font>" );

    return line;
}

QString TeletextWidget::createLinks( const QString &line )
{
    QString result(line);

    QRegExp re( "(\\d\\d\\d)" );
    int pos = re.search(result);
    while ( pos != -1 ) {
	QString s("<a href=\"%1\">%2</a>");
	QString x( s.arg(re.cap(1)).arg(re.cap(1)) );

	result.replace( pos, 3, x );
	pos = re.search( result, pos + x.length() );
    }

    return result;
}

void TeletextWidget::updateGeometry()
{
    QFont f = font();
    f.setFamily( "Courier" );
    f.setPointSize( view->width()/40+1 );
    f.setBold( true );

    header->setFont( f );
    view->setFont( f );
}

void TeletextWidget::resizeEvent( QResizeEvent *e )
{
    QVBox::resizeEvent( e );
    updateGeometry();
}

